/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include "B_comm.h"

extern	int	debug_flag2;

int	send_data(sock_type sock, char *device, DATA_TYPE *value, int count, char typecode, char opcode, int ack, struct timeval *p_tv)
{
	unsigned char	packet[PACKET_SIZE];
	unsigned char	stop;
	int	packet_size, return_packet_size;
	int	packet_count;
	COUNT_TYPE count_in_packet;
	int	i;

	typecode = SEND | (typecode & (~SEND_RECV_MASK));
	if (ack)
		typecode = ACK | (typecode & (~ACK_NOACK_MASK));
	else
		typecode = NOACK | (typecode & (~ACK_NOACK_MASK));

	// send single packet, not stop/continue
	count_in_packet = count * sizeof(DATA_TYPE);

	memcpy(packet, PREAMBLE_CONTENT, PREAMBLE_SIZE);
	packet[TYPECODE_OFFSET] = typecode;
	packet[OPCODE_OFFSET] = opcode;
	memcpy(&packet[COUNT_OFFSET], (char *)&count_in_packet, sizeof(COUNT_TYPE));
	memcpy(&packet[DATA_OFFSET], (char *)value, count_in_packet);

	packet_size = (int)count_in_packet + DATA_OFFSET;

	send_to_socket(sock, device, packet, packet_size);

	if ((typecode & ACK_NOACK_MASK) == ACK)
	{
		return_packet_size = recv_from_socket(sock, device, packet, PACKET_SIZE, p_tv);

		if (return_packet_size < 0)
			return 1;
	}


	/*
	// send multiple packets
	packet_count = (count + ITEM_COUNT_PER_PACKET - 1) / ITEM_COUNT_PER_PACKET;
	for (i = 0; i < packet_count; i ++)
	{
		if (i == packet_count - 1)
		{
			stop = STOP;
			count_in_packet = (COUNT_TYPE)(count - i * ITEM_COUNT_PER_PACKET);
		}
		else
		{
			stop = CONTINUE;
			count_in_packet = (COUNT_TYPE)ITEM_COUNT_PER_PACKET;
		}
		count_in_packet *= sizeof(DATA_TYPE);

		typecode = stop | (typecode & (~STOP_CONTINUE_MASK));

		memcpy(packet, PREAMBLE_CONTENT, PREAMBLE_SIZE);
		packet[TYPECODE_OFFSET] = typecode;
		packet[OPCODE_OFFSET] = opcode;
		memcpy(&packet[COUNT_OFFSET], (char *)&count_in_packet, sizeof(COUNT_TYPE));
		memcpy(&packet[DATA_OFFSET], (char *)&value[i * ITEM_COUNT_PER_PACKET], count_in_packet);

		send_to_socket(sock, device, packet, PACKET_SIZE);

		if ((typecode & ACK_NOACK_MASK) == ACK)
		{
			return_packet_size = recv_from_socket(sock, device, packet, PACKET_SIZE, p_tv);

			if (return_packet_size < 0)
				return 1;

			//if ((packet[TYPECODE_OFFSET] & DATA_TYPE_MASK) != (typecode & DATA_TYPE_MASK)
	            //|| (packet[TYPECODE_OFFSET] & STOP_CONTINUE_MASK) != stop)
		//		return 2;
		}
	}
	*/

	return 0;
}


int	recv_data(sock_type sock, char *device, DATA_TYPE *value, int *count, char typecode, char opcode, int ack, struct timeval *p_tv)
{
	unsigned char	packet[PACKET_SIZE];
	int	return_packet_size;
	int	packet_count;
	COUNT_TYPE count_in_packet;

	// receive single packet, not stop/continue
	typecode = RECV | (typecode & (~SEND_RECV_MASK));
	if (ack)
		typecode = ACK | (typecode & (~ACK_NOACK_MASK));
	else
		typecode = NOACK | (typecode & (~ACK_NOACK_MASK));
	
	memcpy(packet, PREAMBLE_CONTENT, PREAMBLE_SIZE);

	packet[TYPECODE_OFFSET] = typecode;
	packet[OPCODE_OFFSET] = opcode;

	count_in_packet = 0;
	memcpy(&packet[COUNT_OFFSET], (char *)&count_in_packet, sizeof(COUNT_TYPE));

	//send_to_socket(sock, device, packet, PACKET_SIZE);
	send_to_socket(sock, device, packet, DATA_OFFSET);

	return_packet_size = recv_from_socket(sock, device, packet, PACKET_SIZE, p_tv);
	if (return_packet_size < 0)
		return 1;

	memcpy((char *)&count_in_packet, &packet[COUNT_OFFSET], sizeof(COUNT_TYPE));

	if (count_in_packet > *count * sizeof(DATA_TYPE))
	{
		if (debug_flag2)
		{
			printf("Received bogus packet with char count %d > %d\n", (int)count_in_packet, *count * sizeof(DATA_TYPE));
			count_in_packet = *count * sizeof(DATA_TYPE);
		}
		else
			return 1;
	}
		
	memcpy((char *)value, &packet[DATA_OFFSET], count_in_packet);

	*count = (int)count_in_packet / sizeof(DATA_TYPE);

	if ((packet[TYPECODE_OFFSET] & ACK_NOACK_MASK) == ACK)
		send_to_socket(sock, device, packet, ACK_SIZE);

	/*
	// receive multiple packets
	typecode = RECV | (typecode & (~SEND_RECV_MASK));
	typecode = STOP | (typecode & (~STOP_CONTINUE_MASK));
	if (ack)
		typecode = ACK | (typecode & (~ACK_NOACK_MASK));
	else
		typecode = NOACK | (typecode & (~ACK_NOACK_MASK));
	
	memcpy(packet, PREAMBLE_CONTENT, PREAMBLE_SIZE);

	packet[TYPECODE_OFFSET] = typecode;
	packet[OPCODE_OFFSET] = opcode;
	count_in_packet = 0;
	memcpy(&packet[COUNT_OFFSET], (char *)&count_in_packet, sizeof(COUNT_TYPE));

	send_to_socket(sock, device, packet, PACKET_SIZE);

	*count = 0;
	while (1)
	{
		return_packet_size = recv_from_socket(sock, device, packet, PACKET_SIZE, p_tv);
		if (return_packet_size < 0)
			//continue;
			return 1;

	//	if ((packet[TYPECODE_OFFSET] & DATA_TYPE_MASK) != (typecode & DATA_TYPE_MASK))
	//		return 2;

		memcpy((char *)&count_in_packet, &packet[COUNT_OFFSET], sizeof(COUNT_TYPE));

		memcpy((char *)&value[*count], &packet[DATA_OFFSET], (int)count_in_packet);

		*count += (int)count_in_packet / sizeof(DATA_TYPE);

		if ((packet[TYPECODE_OFFSET] & ACK_NOACK_MASK) == ACK)
			send_to_socket(sock, device, packet, ACK_SIZE);

		if ((packet[TYPECODE_OFFSET] & STOP_CONTINUE_MASK) == STOP)
			break;
	}
	*/

	return 0;
}
