/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include "B_ethernet.h"

int debug_flag = 1;



sock_type	open_socket(char *device, double to_s)
{
	sock_type	sock;

#ifdef	_WIN32

	char	error[PCAP_ERRBUF_SIZE];
	int	to_ms;

	to_ms = (int)(to_s * 1000);

	sock = pcap_open_live(device, 65536, 1, to_ms, error);

	if (sock == NULL)
		printf("open_socket(%s) failed: %s\n", device, error);
	else
		if (debug_flag)
			printf("open_socket(%s) done\n", device);

#else

	sock = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));

	if (sock < 0)
		printf("open_socket(): failed\n");
	else
		if (debug_flag)
			printf("open_socket(): done\n");

#endif

	return sock;
}



void	close_socket(sock_type sock)
{
#ifdef	_WIN32

	pcap_close(sock);

#else

	close(sock);

#endif

	return;
}



void	set_blocking(sock_type sock, int blocking)
{	
	int	non_blocking = !blocking;

#ifdef	_WIN32

	/*
	char	error[PCAP_ERRBUF_SIZE];

	if (pcap_setnonblock(sock, non_blocking, error) != 0)
		printf("pcap_setnonblock(%d) failed: %s\n", non_blocking, error);
	else
		if (debug_flag)
			printf("pcap_setnonblock(%d): done\n", non_blocking);
			*/

#else

	ioctl(sock, FIONBIO, &non_blocking);

	/*
	if (blocking)
		fcntl(sock, F_SETFL, O_BLOCK);
	else
		fcntl(sock, F_SETFL, O_NONBLOCK);
		*/

#endif

	return;
}



void set_promiscuous_mode(sock_type sock, char* device, int promiscuous_mode)
{
#ifndef	_WIN32

	//int sock;
	struct ifreq flags;
	
	//sock = socket(PF_INET, SOCK_PACKET, htons(ETH_P_ALL));
	strcpy(flags.ifr_name, device);
	ioctl(sock, SIOCGIFFLAGS, &flags);
	if (promiscuous_mode == 1)
		flags.ifr_flags |= IFF_PROMISC;
	else
		flags.ifr_flags &= ~IFF_PROMISC;
	ioctl(sock, SIOCSIFFLAGS, &flags);
	//close(sock);
#endif

	return;
}



int	send_to_socket(sock_type sock, char *device, char *buffer, int buffer_len)
{
#ifdef	_WIN32

	pcap_sendpacket(sock, buffer, buffer_len);

#else		

	struct sockaddr address;
	
	memset(&address, 0, sizeof(address));
	address.sa_family = AF_INET;
	strncpy(address.sa_data, device, sizeof(address.sa_data));
	sendto(sock, buffer, buffer_len, 0, &address, sizeof(address));

#endif

	if (debug_flag == 1)
		show_packet(buffer, buffer_len, 1);

	return 0;
}




int	recv_from_socket(sock_type sock, char *device, char *buffer, int buffer_len, struct timeval *p_tv)
{
	int packet_len;

#ifdef	_WIN32
	
	int	r;
	struct pcap_pkthdr *header;
	u_char *packet;

	//printf("ready to receive\n");

	r = pcap_next_ex(sock, &header, &packet);

	//printf("receive done, r = %d, packet_len = %d\n", r, header->caplen);

	if (r < 0)
	{
		printf("Error reading the packets: %s\n", pcap_geterr(sock));
		return -1;
	}

	if (r == 0)
	{
		if (debug_flag == 1)
			printf("Nothing read from %s\n", device);
		return 0;
	}

	packet_len = header->caplen;
	if (packet_len > buffer_len)
		packet_len = buffer_len;

	memcpy(buffer, (char *)packet, packet_len);

	if (debug_flag == 1)
		show_packet(buffer, packet_len, 0);

#else

	struct sockaddr address;
	socklen_t address_len;

	fd_set readfds;	

	if (p_tv != NULL)
	{
		FD_ZERO(&readfds);
		FD_SET(sock, &readfds);
		
		select(sock + 1, &readfds, 0, 0, p_tv);
	}

	address_len = sizeof(address);
	packet_len = recvfrom(sock, buffer, buffer_len, 0, &address, &address_len);

	if (packet_len < 0)
	{
		if (debug_flag == 1)
			printf("Nothing read from %s\n", device);
		packet_len = 0;
	}
	else
	{
		if (strcmp(address.sa_data, device) == 0)
		{
			if (debug_flag == 1)
				show_packet(buffer, packet_len, 0);
		}
		else
		{
			printf("Received errand packet from %s\n", address.sa_data);
			// received from another device
			// could messup here

			if (debug_flag == 1)
				show_packet(buffer, packet_len, -1);
			packet_len = -1;
		}
	}

#endif

	return packet_len;
}



void	show_packet(char *packet, int packet_len, int is_send)
{
	int i;
	
	if (is_send == 1)
		printf("\nSent ");
	else if (is_send == 0)
		printf("\nReceived ");
	else if (is_send == -1)
		//return;
		printf("\nReceived junk ");

	printf("\nData packet length = %d\n\n", packet_len);

	if (is_send == -1)
	//	return;
			;

	for (i = 0; i < packet_len; i++)
	{
		printf("%2.2hX ", (unsigned char)packet[i]);
		if (i%16 == 15)
			printf("\n");
	}
	printf("\n");

	return;
}


int	hex_char_to_binary(char c)
{
	int result;

	if (isxdigit(c))
	{
		if (isdigit(c))
			result = c - '0';
		else
			result = toupper(c) - 'A' + 10;
	}
	else if (c != 0)
	{
		printf("non hex char: %c\n");
		return -1;
	}

	return result;
}


void interrupt_handler(int i)
{
	extern	char	*device;
	extern	sock_type sock;

	set_blocking(sock, 1);
	set_promiscuous_mode(sock, device, 0);
	close_socket(sock);
	
	exit(0);
}
