##	Copyright (c) 2004 The Regents of the University of California. All Rights
##	Reserved. Permission to use, copy, modify, and distribute this  software
##	and its documentation for educational, research and non-profit purposes,
##	without fee, and without a written agreement is hereby granted, provided
##	that the above copyright notice, this paragraph and the following two
##	paragraphs appear in all copies. This software program and documentation
##	are copyrighted  by The Regents of the University of California ("The
##	University of California").
##
##	THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
##	ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
##	UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
##	PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
##	THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
##	TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.
##
##	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
##	DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
##	LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
##	DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
##	THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
##	DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##	WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
##	SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
##	CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
##	ENHANCEMENTS, OR  MODIFICATIONS.

import	os
import	operator
import	copy
import	time

pro_trace_ext = '.pro_trace'
emu_trace_ext = '.emu_trace'
sim_trace_ext = '.sim_trace'

label_txt = '_txt'
label_gui = '_gui'

points_per_second = 10

def	get_filename():
	return time.ctime().replace(' ', '_').replace(':', '-')


def	get_pro_filename(suffix = ''):
	return get_filename() + suffix + pro_trace_ext


def	get_emu_filename(suffix = ''):
	return get_filename() + suffix + emu_trace_ext


def	get_sim_filename(suffix = ''):
	return get_filename() + suffix + sim_trace_ext


def	convertTrace(file1, file2, msg = 0):
	if msg:
		print 'Converting trace file:', file1, 'to:', file2
	
	f = open(file1)
	lines = f.readlines()
	f.close()

	t = []
	c = []
	v = []
	
	temp_file = 'tempfile.txt'
	f = open(temp_file, 'w')

	for line in lines:
		if line.startswith('#'):
			continue
		else:
			if line.strip() == '':
				continue
			item = line.split()
			t.append(float(item[0]))
			c.append(float(item[1]))
			v.append(float(item[2]))
			
			print >> f, line.strip()

	f.close()

	n = len(t)

	e, iv, ic = computeVars(t, c, v)

	f = open(temp_file)
	s = f.read()
	f.close()

	f = open(file2, 'w')
	print >> f, '# n = ', n
	print >> f, s
	print >> f, '# e = ', e
	print >> f, '#', 'v0 = ', v[0]
	print >> f, '#', 'v1 = ', v[-1]
	print >> f, '# iv = ', iv
	print >> f, '# ic = ', ic

	if file1 == file2:
		print >> f, '# Reconstructed from:', file1
	else:
		print >> f, '# Converted from:', file1
	print >> f
	f.close()
	
	if msg:
		print 'Results written to:', file2
	
	return t, c, v, e, iv, ic


def	convertLongProfilingTrace(file1, file2, msg = 1):
	if msg:
		print 'Converting long profiling trace file:', file1, 'to:', file2
	
	f = open(file1)
	lines = f.readlines()
	f.close()

	t = []
	c = []
	v = []
	
	temp_file = 'tempfile.txt'
	f = open(temp_file, 'w')

	last_time = 0
	c_ = []
	v_ = []

	for line in lines:
		if line.startswith('#'):
			continue
		elif line.startswith('!'):
			c.extend(c_)
			v.extend(v_)
			item = line.split()
			real_time = float(item[1])
			delay = float(item[2])
			n_ = int(item[3])
			time_step = delay / n_
			for i in range(n_):
				t_ = last_time + i * time_step
				t.append(t_)
				print >> f, '%12.6f' % t_, '\t%12.6f' % c_[i], '\t%12.6f' % v_[i]
			last_time = real_time
			c_ = []
			v_ = []
		else:
			item = line.split()
			c_.append(float(item[0]))
			v_.append(float(item[1]))
			
	f.close()

	n = len(t)
	if n != len(c) or n != len(v):
		print 't:', n, 'c:', len(c), 'v:', len(v)
		raise 'error'

	e, iv, ic = computeVars(t, c, v)
	
	f = open(temp_file)
	s = f.read()
	f.close()

	f = open(file2, 'w')
	print >> f, '# n = ', n
	print >> f, s
	print >> f, '# e = ', e
	print >> f, '# iv = ', iv
	print >> f, '# ic = ', ic
	
	print >> f, '# Converted from:', file1
	f.close()

	if msg:
		print 'Profiling results written to:', file2

	return	t, c, v, e, iv, ic


def	convertShortProfilingTrace(file1, file2, msg = 1):
	if msg:
		print 'Converting short profiling trace file:', file1, 'to:', file2
	
	r =  convertTrace(file1, file2, msg = 0)

	if msg:
		print 'Profiling results written to:', file2
		
	return r


def	convertEmulationTrace(file1, file2, msg = 1):
	if msg:
		print 'Converting emulation trace file:', file1, 'to:', file2

	r =  convertTrace(file1, file2, msg = 0)

	if msg:
		print 'Emulation results written to:', file2
	
	return r


def	convertSimulationTrace(file1, file2, msg = 1):
	if msg:
		print 'Converting simulation trace file:', file1, 'to:', file2
	
	r =  convertTrace(file1, file2, msg = 0)

	if msg:
		print 'Simulation results written to:', file2
		
	return r
	

def	convertOldTraceFile(file1, file2, msg = 1):
	if msg:
		print 'Converting old trace file:', file1, 'to:', file2
	f = open(file1)
	s = f.read()
	f.close()
	lines = s.splitlines()

	t = []
	c = []
	v = []
	
	while lines[-1].strip() == '':
		lines.pop()
	
	n = len(lines)

	last_item = lines[-1].split()
	t_max = float(last_item[0])

	#if n > t_max * points_per_second:
	#	stride = int(n / t_max / points_per_second)
	#else:
	#	stride = 1

	stride = 1

	i = 0
	while i < n:
		line= lines[i]
		item = line.split()
		t.append(float(item[0]))
		c.append(float(item[1]))
		v.append(float(item[2]))
		i += stride

	n = len(t)
	
	e, iv, ic = computeVars(t, c, v)
	
	f = open(file2, 'w')
	print >> f, '# n = ', n
	for i in range(n):
		print >> f, '\t%12.6f' % t[i], '\t%12.6f' % c[i], '\t%12.6f' % v[i]
	#print >> f, s
	print >> f, '# e = ', e
	print >> f, '#', 'v0 = ', v[0]
	print >> f, '#', 'v1 = ', v[-1]
	print >> f, '# iv = ', iv
	print >> f, '# ic = ', ic

	print >> f, '# Converted from:', file1
	f.close()

	return	t, c, v, e, iv, ic


def	parseTraceFile(file):
	t = []
	c = []
	v = []
	lines = open(file).readlines()

	if not lines[0].startswith('#'):
		file1 = file + '_old'
		if os.path.exists(file1):
			os.remove(file1)
		os.rename(file, file1)
		return convertOldTraceFile(file1, file)
	else:
		n = int(lines[0].split()[-1])
		t = []
		c = []
		v = []

		lines.pop(0)

		last_item = lines[n - 1].split()
		t_max = float(last_item[0])

		#if n > t_max * points_per_second:
		#	stride = int(n / t_max / points_per_second)
		#else:
		#	stride = 1

		stride = 1

		i = 0
		while i < n:
			line= lines[i]
			item = line.split()
			t.append(float(item[0]))
			c.append(float(item[1]))
			v.append(float(item[2]))
			i += stride

		for line in lines[n:]:
			s = line.strip()
			if s == '':
				continue
			s = line[1:]
			s = s.strip()
			#print s
			try:
				exec s
			except:
				pass

	return t, c, v, e, iv, ic
		

def	mergeTime(t1, t2):
	p1 = len(t1)
	p2 = len(t2)
	i = 0
	j = 0
	t = []
	while 1:
		if t1[i] < t2[j]:
			t.append(t1[i])
			i += 1
		elif t1[i] > t2[j]:
			t.append(t2[j])
			j += 1
		else:
			t.append(t1[i])
			i += 1
			j += 1
		if i == p1:
			t.extend(t2[j:])
			break
		if j == p2:
			t.extend(t1[i:])
			break
	return t


def	addPoint(t, t1, x1):
	n = len(t)
	n1 = len(t1)
	
	i = 0
	j = 0
	x = []

	while 1:
		if t[i] < t1[j]:
			x.append(x1[j - 1])
##			if j > 0:
##				x.append(x1[j - 1])
##			else:
##				x.append(x1[j])
			i += 1
		elif t[i] == t1[j]:
			x.append(x1[j])
			i += 1
			j += 1
		else:
			j += 1
		if j == n1:
			x_last = x[-1]
			while i < n:
				x.append(x_last)
				i += 1
			break
		if i == n:
			break

	if len(x) != n:
		print 'error', 't:', n, 'x:', len(x)
		return None

	return x
	
	
	

	
def	compareTraceFile(file1, file2):
	t1, c1, v1, e1, iv1, ic1 = parseTraceFile(file1)
	t2, c2, v2, e2, iv2, ic2 = parseTraceFile(file2)

	t = mergeTime(t1, t2)

	v_1 = addPoint(t, t1, v1)
	v_2 = addPoint(t, t2, v2)

	c_1 = addPoint(t, t1, c1)
	c_2 = addPoint(t, t2, c2)

	p_1 = map(operator.mul, c_1, v_1)
	p_2 = map(operator.mul, c_2, v_2)
	
	dv = map(operator.sub, v_1, v_2)
	dv_abs = map(operator.abs, dv)
	
	dc = map(operator.sub, c_1, c_2)
	dc_abs = map(operator.abs, dc)
	
	dp = map(operator.sub, p_1, p_2)
	dp_abs = map(operator.abs, dp)

	i_dv = integral(dv, t)
	i_dc = integral(dc, t)
	i_dp = integral(dp, t)

	i_dv_abs = integral(dv_abs, t)
	i_dc_abs = integral(dc_abs, t)
	i_dp_abs = integral(dp_abs, t)

	err_v = i_dv / iv2
	err_c = i_dc / ic2
	err_p = i_dp / e2

	err_v_abs = i_dv_abs / iv2
	err_c_abs = i_dc_abs / ic2
	err_p_abs = i_dp_abs / e2

	err_e = e1 / e2 - 1

	f = open(file1, 'a+')
	print >> f, '#', 'file_ref = ', '\'' + file2 +'\''
	print >> f, '#', 'err_e = ', err_e
	print >> f, '#', 'err_v = ', err_v
	print >> f, '#', 'err_c = ', err_c
	print >> f, '#', 'err_p = ', err_p
	print >> f, '#', 'err_v_abs = ', err_v_abs
	print >> f, '#', 'err_c_abs = ', err_c_abs
	print >> f, '#', 'err_p_abs = ', err_p_abs
	print >> f
	f.close()


def	integral(x, t):
	t_ = t[:]
	dt = map(operator.sub, t[1:], t_[:-1])
	x_dt = map(operator.mul, x[:-1], dt)
	e = reduce(operator.add, x_dt, 0.)
	return e
	

def	computeVars(t, c, v):
	p = map(operator.mul, c, v)
	e = integral(p, t)
	iv = integral(v, t)
	ic = integral(c, t)

	return e, iv, ic


def	extractComparedParameter(file, file_ref0, par_list):
	f = open(file)
	lines = f.readlines()
	f.close()

	n = int(lines[0].split()[-1])

	info_lines = lines[n:]

	info_lines.reverse()

	file_ref = ''
	for line in info_lines:
		line = line.replace('#', '').strip()
		if line == '':
			continue
		try:
			exec line
		except:
			pass

		if file_ref == file_ref0:
			break

	retvar_list = []
	
	for par in par_list:
		try:
			v = eval(par)
		except:
			v = None
		retvar_list.append(v)

##	if retvar_list.count(None) == len(retvar_list):
##		return None
##	else:
##		return retvar_list
	return retvar_list

def	extractDataParameter(file, par_list):
	f = open(file)
	lines = f.readlines()
	f.close()

	n = int(lines[0].split()[-1])

	info_lines = lines[n:]

	file_ref0 = ''
	file_ref = ''
	for line in info_lines:
		line = line.replace('#', '').strip()
		if line == '':
			continue
		try:
			exec line
		except:
			pass

		if file_ref != file_ref0:
			break

	retvar_list = []
	
	for par in par_list:
		try:
			v = eval(par)
		except:
			v = None
		retvar_list.append(v)

##	if retvar_list.count(None) == len(retvar_list):
##		return None
##	else:
##		return retvar_list
	return retvar_list


def	rebuildDataInfo(file):
	
	convertTrace(file, file, msg = 0)
