/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include <stdio.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <stdlib.h>

#include "B_dualfoil.h"
#include "B_comm.h"
#include "B_dataexcg.h"
#include "B_func.h"

extern	int	ack;
extern	int	debug_flag2;
extern	int	max_tick;
extern	double	sim_interval, elapsed_time;

extern	float	cu0;

sock_type	sock;

char	device[100] = "eth0";
int	blocking = 1;

int	done = 0;

int	r_dualfoil;

extern	int	parameter_ok;
int	network_ok = 0;


extern	double	sim_voltage[], sim_time[];
extern	int	sim_count;

//double	time_out;

void my_interrupt_handler(int i)
{
	if (done == 1)
		exit(1);

	stop_network();

	done = 1;

	//exit(1);
}


int	start_network(void)
{
	if (parameter_ok == 0)
		init_parameter();

	sock = open_socket(device, sim_interval);
	set_blocking(sock, blocking);
	set_promiscuous_mode(sock, device, 1);

	signal(SIGINT, my_interrupt_handler);

	network_ok = 1;

	return 0;
}


void	stop_network(void)
{
	set_promiscuous_mode(sock, device, 0);
	close_socket(sock);
	//done = 1;

	network_ok = 0;

	return;
}


void	init_emulation(void)
{
	int	r;

	if (parameter_ok == 0)
		init_parameter();
	//time_out = sim_interval;

	if (network_ok == 0)
		start_network();


	if ((r = init_simulation()) != 0)
	{
		printf("Error: init_simulation() failed with return_code: %d\n", r);
		exit(3);
	}

	if ((r = init_board(sock, device, NOP)) != 0)
	{
		printf("Error: init_board() failed with return_code: %d\n", r);
		exit(3);
	}

	if ((r = send_voltage(sock, device, MEASURE_AVG_CURRENT | EMULATE_VOLTAGE)) != 0)
		printf("Error: send_voltage() failed with return_code: %d\n", r);


	return;
}


int	run_emulation_step(void)
{
	int	r;
	float	tt0;
	int	mc0 = 1;
	int	new_value = 1;

	time_t	t1, t2;
	clock_t c1, c2;

	tt0 = sim_interval / 60;

	c1 = clock();
	time(&t1);

	if (debug_flag2)
		printf("simulate_next_step(%d, %f, %f, %d), r = %d\n", new_value, cu0, tt0, mc0, r);

	r_dualfoil = simulate_next_step(new_value, (double)cu0, (double)tt0, mc0);

	time(&t2);
	c2 = clock();

	if (debug_flag2)
		printf("Actual time: %f, simulation time: %f, CPU clock: %f\n", tt0, difftime(t2, t1), (double)(c2 - c1) / CLOCKS_PER_SEC);

	if (r_dualfoil != 53)
	{
		done = 1;
		return 1;
	}

	if ((r = send_voltage(sock, device, NOP)) != 0)
	{
		if (done == 1)
			return 1;
		printf("Error: send_voltage() failed with return_code: %d\n", r);
	}

	if ((r = recv_avg_current(sock, device, MEASURE_AVG_CURRENT | EMULATE_VOLTAGE)) != 0)
	{
		if (done == 1)
			return 1;
		printf("Error: recv_avg_current() failed with return_code: %d\n", r);
	}


	elapsed_time += sim_interval;

	return done;
}


int	test_emu_voltage(double v)
{
	int	r;

	if (debug_flag2)
		printf("set_voltage(%f)\n", v);

	sim_voltage[0] = v;
	sim_voltage[1] = v + 0.00001;
	sim_time[0] = 0.;
	sim_time[1] = sim_interval;
	sim_count = 2;

	if ((r = send_voltage(sock, device, MEASURE_AVG_CURRENT | EMULATE_VOLTAGE)) != 0)
	{
		if (done == 1)
			return 1;
		printf("Error: send_voltage() failed with return_code: %d\n", r);
	}

	return 0;
}




void	complete_emulation(void)
{
	network_ok = parameter_ok = 0;

	stop_network();
	//complete_simulation(r_dualfoil);
	return;
}


void	run_emulation(void)
{
	int	r;
	float	tt0;
	int	mc0 = 1;
	int	new_value = 1;

	time_t	t1, t2;
	clock_t c1, c2;

	tt0 = sim_interval / 60;

	while (!done) 
	{
		r = run_emulation_step();

		if (r != 0)
			break;
	}

	complete_emulation();
	return;
}


void	init_profiling(void)
{
	int	r;

	if (parameter_ok == 0)
		init_parameter();
	//time_out = 0.001;

	if (network_ok == 0)
		start_network();

	if ((r = init_board(sock, device, MEASURE_CURRENT | MEASURE_VOLTAGE)) != 0)
	{
		printf("Error: init_board() failed with return_code: %d\n", r);
		exit(3);
	}

	return;
}


int	run_profiling_step(void)
{
	int	r;

	/*
	if ((r = recv_current(sock, device, NOP)) != 0)
		printf("Error: recv_current() failed with return_code: %d\n", r);

	if ((r = recv_voltage(sock, device, MEASURE_CURRENT | MEASURE_VOLTAGE)) != 0)
		printf("Error: recv_voltage() failed with return_code: %d\n", r);
	*/
	
	// blind receive voltage and current
	if ((r = recv_voltage_current(sock, device, NOP)) != 0)
	{
		if (done)
			return 1;
		printf("Error: recv_voltage_current() failed with return_code: %d\n", r);
	}

	elapsed_time += sim_interval;

	return done;
}


void	run_profiling(void)
{
	while (!done) 
		run_profiling_step();

	return;
}


void	complete_profiling(void)
{
	network_ok = parameter_ok = 0;

	stop_network();
	return;
}
