/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include <string.h>
#include <Python.h>

#include "B_func.h"
#include "B_dualfoil.h"


#define	RETURN_NONE	\
	{Py_INCREF(Py_None);	return Py_None;}


#define	RETURN_STR(py_obj, value)		\
	return Py_BuildValue("s#", value, strlen(value));

#define	RETURN_INT(value)	\
	return Py_BuildValue("i", value);

#define	RETURN_INT2(value1, value2)	\
	return Py_BuildValue("(ii)", value1, value2);


#define	RETURN_FLT(value)	\
	return Py_BuildValue("d", value);

#define	RETURN_FLT2(value1, value2)	\
	return Py_BuildValue("(dd)", value1, value2);


//#define	PARSE_NONE(args)		\
//	PyArg_ParseTuple(args, "|");


#define	PARSE_STR(args, value)		\
	if (PyArg_ParseTuple(args, "s", &value) == 0)	\
		RETURN_NONE	

#define	PARSE_INT(args, value)		\
	if (PyArg_ParseTuple(args, "i", &value) == 0)	\
		RETURN_NONE	


#define	PARSE_INT2(args, value1, value2)		\
	if (PyArg_ParseTuple(args, "ii", &value1, &value2) == 0)	\
		RETURN_NONE

#define	PARSE_INT3(args, value1, value2, value3)		\
	if (PyArg_ParseTuple(args, "iii", &value1, &value2, &value3) == 0)	\
		RETURN_NONE


#define	PARSE_FLT(args, value)		\
	if (PyArg_ParseTuple(args, "d", &value) == 0)	\
		RETURN_NONE

#define	PARSE_FLT2(args, value1, value2)	\
	if (PyArg_ParseTuple(args, "dd", &value1, &value2) == 0)	\
		RETURN_NONE


#define	PARSE_FLT_INT(args, value1, value2)		\
	if (PyArg_ParseTuple(args, "di", &value1, &value2) == 0)	\
		RETURN_NONE

#define	PARSE_FLT2_INT(args, value1, value2, value3)		\
	if (PyArg_ParseTuple(args, "ddi", &value1, &value2, &value3) == 0)	\
		RETURN_NONE

#define	BUILD_INT(py_obj, value)		\
	py_obj = Py_BuildValue("i", value);

#define	BUILD_FLT(py_obj, value)		\
	py_obj = Py_BuildValue("d", value);

#define	BUILD_STR(py_obj, value)		\
	py_obj = Py_BuildValue("s#", value, strlen(value));


extern	char	device[];
extern	int	blocking;
extern	double	sim_interval;
extern	double	tick_interval;
extern	int	max_tick;
extern	double	elapsed_time;

extern	int	debug_flag;
extern	int	debug_flag2;
extern	int	B_debug_flag1;
extern	int	B_debug_flag2;

extern	double	mea_avg_current;

extern	double	mea_current[];
extern	int	mea_current_count;

extern	double	mea_voltage[];
extern	int	mea_voltage_count;

extern	double	sim_voltage[];
extern	double	sim_time[];
extern	int	sim_count;

extern	double	emu_voltage[];
extern	double	emu_time[];
extern	int	emu_voltage_count;

extern	double	vref_adc;
extern	double	vref_dac;

extern	double	rc_adc;

extern	double	voffset_adc;
extern	double	voffset_dac;
extern	double	r_vreg;
extern	double	coffset_adc;


static	PyObject	*py_interface;
static	PyObject	*py_blocking;
static	PyObject	*py_sim_interval;
static	PyObject	*py_tick_interval;
static	PyObject	*py_max_tick;
static	PyObject	*py_elapsed_time;

static	PyObject	*py_debug_flag;
static	PyObject	*py_debug_flag2;
static	PyObject	*py_B_debug_flag1;
static	PyObject	*py_B_debug_flag2;


static	PyObject	*py_mea_avg_current;

static	PyObject	*py_mea_current;
static	PyObject	*py_mea_current_count;

static	PyObject	*py_mea_voltage;
static	PyObject	*py_mea_voltage_count;

static	PyObject	*py_sim_voltage;
static	PyObject	*py_sim_time;
static	PyObject	*py_sim_count;

static	PyObject	*py_emu_voltage;
static	PyObject	*py_emu_time;
static	PyObject	*py_emu_voltage_count;

static	PyObject	*py_vref_adc;
static	PyObject	*py_vref_dac;
static	PyObject	*py_rc_adc;

static	PyObject	*py_voffset_adc;
static	PyObject	*py_voffset_dac;
static	PyObject	*py_r_vreg;
static	PyObject	*py_coffset_adc;


void	update_c_parameter(void)
{
	strcpy(device, PyString_AsString(py_interface));
	blocking = PyInt_AsLong(py_blocking);
	sim_interval = PyFloat_AsDouble(py_sim_interval);
	tick_interval = PyFloat_AsDouble(py_tick_interval);
	max_tick = PyInt_AsLong(py_max_tick);
	elapsed_time = PyFloat_AsDouble(py_elapsed_time);

}


static  PyObject * update_py_parameter(void)
{
	BUILD_STR(py_interface, device)
	BUILD_INT(py_blocking, blocking)
	BUILD_FLT(py_sim_interval, sim_interval)
	BUILD_FLT(py_tick_interval, tick_interval)
	BUILD_INT(py_max_tick, max_tick)
	BUILD_FLT(py_elapsed_time, elapsed_time)

	RETURN_NONE
}


static  PyObject * build_py_flt_array(PyObject **py_array, double *c_array, int array_len)
{
	int	i;
	PyObject *temp;

	//PyObject_Del(*py_array);

	*py_array = PyList_New(array_len);
	//Py_INCREF(*py_array);
	for (i = 0; i < array_len; i ++)
	{
		BUILD_FLT(temp, c_array[i])
		PyList_SetItem(*py_array, i, temp);
		//Py_INCREF(temp);
	}

	RETURN_NONE
}



static PyObject	* py_set_interface(PyObject *self, PyObject *args)
{
	char	*s;

	PARSE_STR(args, s)
	strcpy(device, s);
	BUILD_STR(py_interface, device)
	RETURN_NONE
}


static PyObject	* py_get_interface(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_interface);
	return py_interface;
}


static PyObject	* py_set_blocking(PyObject *self, PyObject *args)
{
	PARSE_INT(args, blocking)
	BUILD_INT(py_blocking, blocking)
	RETURN_NONE
}


static PyObject	* py_get_blocking(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_blocking);
	return py_blocking;
}


static PyObject	* py_set_sim_interval(PyObject *self, PyObject *args)
{
	PARSE_FLT(args, sim_interval)
	BUILD_FLT(py_sim_interval, sim_interval)
	RETURN_NONE
}


static PyObject	* py_get_sim_interval(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_sim_interval);
	return py_sim_interval;
}


static PyObject	* py_set_tick_interval(PyObject *self, PyObject *args)
{
	PARSE_FLT(args, tick_interval)
	BUILD_FLT(py_tick_interval, tick_interval)
	RETURN_NONE
}


static PyObject	* py_get_tick_interval(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_tick_interval);
	return py_tick_interval;
}


static PyObject	* py_set_max_tick(PyObject *self, PyObject *args)
{
	PARSE_INT(args, max_tick)
	BUILD_INT(py_max_tick, max_tick)
	RETURN_NONE
}


static PyObject	* py_get_max_tick(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_max_tick);
	return py_max_tick;
}


static PyObject	* py_set_elapsed_time(PyObject *self, PyObject *args)
{
	PARSE_FLT(args, elapsed_time)
	BUILD_FLT(py_elapsed_time, elapsed_time)
	RETURN_NONE
}


static PyObject	* py_get_elapsed_time(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_elapsed_time);
	return py_elapsed_time;
}


static PyObject	* py_set_debug_flag1(PyObject *self, PyObject *args)
{
	PARSE_INT(args, debug_flag)
	BUILD_INT(py_debug_flag, debug_flag)
	RETURN_NONE
}


static PyObject	* py_get_debug_flag1(PyObject *self, PyObject *args)
{
	Py_INCREF(py_debug_flag);
	return py_debug_flag;
}


static PyObject	* py_set_debug_flag2(PyObject *self, PyObject *args)
{
	PARSE_INT(args, debug_flag2)
	BUILD_INT(py_debug_flag2, debug_flag2)
	RETURN_NONE
}


static PyObject	* py_get_debug_flag2(PyObject *self, PyObject *args)
{
	Py_INCREF(py_debug_flag2);
	return py_debug_flag2;
}


static PyObject	* py_set_B_debug_flag1(PyObject *self, PyObject *args)
{
	PARSE_INT(args, B_debug_flag1)
	BUILD_INT(py_B_debug_flag1, B_debug_flag1)
	RETURN_NONE
}



static PyObject	* py_get_B_debug_flag1(PyObject *self, PyObject *args)
{
	Py_INCREF(py_B_debug_flag1);
	return py_B_debug_flag1;
}


static PyObject	* py_set_B_debug_flag2(PyObject *self, PyObject *args)
{
	PARSE_INT(args, B_debug_flag2)
	BUILD_INT(py_B_debug_flag2, B_debug_flag2)
	RETURN_NONE
}


static PyObject	* py_get_B_debug_flag2(PyObject *self, PyObject *args)
{
	Py_INCREF(py_B_debug_flag2);
	return py_B_debug_flag2;
}


static PyObject	* py_get_sim_voltage(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_sim_voltage);
	return py_sim_voltage;
}


static PyObject	* py_get_sim_time(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_sim_time);
	return py_sim_time;
}


static PyObject	* py_get_emu_voltage(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_emu_voltage);
	return py_emu_voltage;
}


static PyObject	* py_get_emu_time(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_emu_time);
	return py_emu_time;
}


static PyObject	* py_get_mea_voltage(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_mea_voltage);
	return py_mea_voltage;
}


static PyObject	* py_get_mea_current(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_mea_current);
	return py_mea_current;
}


static PyObject	* py_get_mea_avg_current(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_mea_avg_current);
	return py_mea_avg_current;
}


static PyObject	* py_init_parameter(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	update_c_parameter();
	init_parameter();
	update_py_parameter();
	RETURN_NONE
}


static PyObject	* py_init_emulation(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	update_c_parameter();
	init_emulation();
	update_py_parameter();
	RETURN_NONE
}


static PyObject	* py_run_emulation_step(PyObject *self, PyObject *args)
{
	int	r;

	//PARSE_NONE(args)

	r = run_emulation_step();

	BUILD_FLT(py_mea_avg_current, mea_avg_current)

	BUILD_INT(py_sim_count, sim_count)
	build_py_flt_array(&py_sim_voltage, sim_voltage, sim_count);
	build_py_flt_array(&py_sim_time, sim_time, sim_count);

	BUILD_INT(py_emu_voltage_count, emu_voltage_count)
	build_py_flt_array(&py_emu_voltage, emu_voltage, emu_voltage_count);
	build_py_flt_array(&py_emu_time, emu_time, emu_voltage_count);

	BUILD_FLT(py_elapsed_time, elapsed_time)

	RETURN_INT(r)
}


static PyObject	* py_complete_emulation(PyObject *self, PyObject *args)
{
	complete_emulation();
	RETURN_NONE
}
	

static PyObject	* py_init_profiling(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)

	update_c_parameter();
	init_profiling();
	update_py_parameter();
	RETURN_NONE
}


static PyObject	* py_run_profiling_step(PyObject *self, PyObject *args)
{
	int	r;

	//PARSE_NONE(args)

	r = run_profiling_step();
	
	BUILD_INT(py_mea_voltage_count, mea_voltage_count)
	build_py_flt_array(&py_mea_voltage, mea_voltage, mea_voltage_count);

	BUILD_INT(py_mea_current_count, mea_current_count)
	build_py_flt_array(&py_mea_current, mea_current, mea_current_count);

	BUILD_FLT(py_elapsed_time, elapsed_time)

	RETURN_INT(r)
}


static PyObject	* py_complete_profiling(PyObject *self, PyObject *args)
{
	complete_profiling();
	RETURN_NONE
}


static	PyObject * py_test_emu_voltage(PyObject *self, PyObject *args)
{
	int	r;
	double	v;

	PARSE_FLT(args, v)
	r = test_emu_voltage(v);
	if (r != 0)
		RETURN_NONE
	else
	{
		v = emu_voltage[0];
		RETURN_FLT(v)
	}
}


static	PyObject * py_start_network(PyObject *self, PyObject *args)
{
	int	r;

	r = start_network();

	if (r != 0)
		RETURN_NONE
	else
		RETURN_INT(0)
}

	
static	PyObject * py_stop_network(PyObject *self, PyObject *args)
{

	stop_network();

	RETURN_NONE
}

static PyObject	* py_init_simulation(PyObject *self, PyObject *args)
{
	init_simulation();
	RETURN_NONE
}


static PyObject	* py_simulate_next_step(PyObject *self, PyObject *args)
{
	int	r;
	double cu0;
	double tt0;
	int mc0;

	PARSE_FLT2_INT(args, cu0, tt0, mc0)

	r = simulate_next_step(1, cu0, tt0, mc0);

	BUILD_INT(py_sim_count, sim_count)
		build_py_flt_array(&py_sim_voltage, sim_voltage, sim_count);
	build_py_flt_array(&py_sim_time, sim_time, sim_count);

	RETURN_INT(r)
}


static PyObject	* py_complete_simulation(PyObject *self, PyObject *args)
{
	int	r;

	PARSE_INT(args, r)
		complete_simulation(r);
	RETURN_NONE
}


static PyObject	* py_set_vref_adc(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, vref_adc)
	BUILD_FLT(py_vref_adc, vref_adc)

	RETURN_NONE
}


static PyObject	* py_get_vref_adc(PyObject *self, PyObject *args)
{

	Py_INCREF(py_vref_adc);
	return py_vref_adc;
}


static PyObject	* py_set_vref_dac(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, vref_dac)
	BUILD_FLT(py_vref_dac, vref_dac)

	RETURN_NONE
}


static PyObject	* py_get_vref_dac(PyObject *self, PyObject *args)
{

	Py_INCREF(py_vref_dac);
	return py_vref_dac;
}


static PyObject	* py_set_rc_adc(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, rc_adc)
	BUILD_FLT(py_rc_adc, rc_adc)

	RETURN_NONE
}


static PyObject	* py_get_rc_adc(PyObject *self, PyObject *args)
{

	Py_INCREF(py_rc_adc);
	return py_rc_adc;
}


static PyObject	* py_set_voffset_adc(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, voffset_adc)
	BUILD_FLT(py_voffset_adc, voffset_adc)

	RETURN_NONE
}


static PyObject	* py_get_voffset_adc(PyObject *self, PyObject *args)
{

	Py_INCREF(py_voffset_adc);
	return py_voffset_adc;
}


static PyObject	* py_set_voffset_dac(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, voffset_dac)
	BUILD_FLT(py_voffset_dac, voffset_dac)

	RETURN_NONE
}


static PyObject	* py_get_voffset_dac(PyObject *self, PyObject *args)
{

	Py_INCREF(py_voffset_dac);
	return py_voffset_dac;
}


static PyObject * py_set_coffset_adc(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, coffset_adc)
	BUILD_FLT(py_coffset_adc, coffset_adc)
	
	RETURN_NONE
}


static PyObject * py_get_coffset_adc(PyObject *self, PyObject *args)
{

	Py_INCREF(py_coffset_adc);
	return py_coffset_adc;
}



static PyObject	* py_set_r_vreg(PyObject *self, PyObject *args)
{

	PARSE_FLT(args, r_vreg)
	BUILD_FLT(py_r_vreg, r_vreg)

	RETURN_NONE
}


static PyObject	* py_get_r_vreg(PyObject *self, PyObject *args)
{

	Py_INCREF(py_r_vreg);
	return py_r_vreg;
}


void	build_variable(void)
{
	int	i;
	PyObject	*temp;

	strcpy(device, "eth0");
	blocking = 1;
	sim_interval = 0.3;
	max_tick = 20;
	tick_interval = sim_interval / max_tick;
	elapsed_time= 0.;

	update_py_parameter();

	debug_flag = 1;
	debug_flag2 = 1;
	B_debug_flag1 = 1;
	B_debug_flag2 = 1;

	BUILD_INT(py_debug_flag, debug_flag)
	BUILD_INT(py_debug_flag2, debug_flag2)
	BUILD_INT(py_B_debug_flag1, B_debug_flag1)
	BUILD_INT(py_B_debug_flag2, B_debug_flag2)

	BUILD_FLT(py_vref_adc, vref_adc)
	BUILD_FLT(py_vref_dac, vref_dac)
	BUILD_FLT(py_rc_adc, rc_adc)

	BUILD_FLT(py_voffset_adc, voffset_adc)
	BUILD_FLT(py_voffset_dac, voffset_dac)
	BUILD_FLT(py_coffset_adc, coffset_adc)
	BUILD_FLT(py_r_vreg, r_vreg)


	mea_avg_current = 0.;
	BUILD_FLT(py_mea_avg_current, mea_avg_current)

	mea_current_count = 0;
	BUILD_INT(py_mea_current_count, mea_current_count)
	build_py_flt_array(&py_mea_current, mea_current, mea_current_count);

	mea_voltage_count = 0;
	BUILD_INT(py_mea_voltage_count, mea_voltage_count)
	build_py_flt_array(&py_mea_voltage, mea_voltage, mea_voltage_count);

	sim_count = 0;
	BUILD_INT(py_sim_count, sim_count)
	build_py_flt_array(&py_sim_voltage, sim_voltage, sim_count);
	build_py_flt_array(&py_sim_time, sim_time, sim_count);

	emu_voltage_count = 0;
	BUILD_INT(py_emu_voltage_count, emu_voltage_count)
	build_py_flt_array(&py_emu_voltage, emu_voltage, emu_voltage_count);
	build_py_flt_array(&py_emu_time, emu_time, emu_voltage_count);

	return;
}


static PyMethodDef B_sharpMethods[] = {
	{"start_network",         py_start_network,                METH_VARARGS},
	{"stop_network",          py_stop_network,                 METH_VARARGS},
	{"init_parameter",        py_init_parameter,               METH_VARARGS},
	{"init_profiling",        py_init_profiling,               METH_VARARGS},
	{"run_profiling_step",    py_run_profiling_step,           METH_VARARGS},
	{"complete_profiling",    py_complete_profiling,           METH_VARARGS},
	{"init_emulation",        py_init_emulation,               METH_VARARGS},
	{"run_emulation_step",    py_run_emulation_step,           METH_VARARGS},
	{"complete_emulation",    py_complete_emulation,           METH_VARARGS},
	{"init_simulation",       py_init_simulation,              METH_VARARGS},
	{"simulate_next_step",    py_simulate_next_step,           METH_VARARGS},
	{"complete_simulation",   py_complete_simulation,          METH_VARARGS},
	{"set_interface",         py_set_interface,                METH_VARARGS},
	{"get_interface",         py_get_interface,                METH_VARARGS},
	{"set_blocking",          py_set_blocking,                 METH_VARARGS},
	{"get_blocking",          py_get_blocking,                 METH_VARARGS},
	{"set_sim_interval",      py_set_sim_interval,             METH_VARARGS},
	{"get_sim_interval",      py_get_sim_interval,             METH_VARARGS},
	{"set_tick_interval",     py_set_tick_interval,            METH_VARARGS},
	{"get_tick_interval",     py_get_tick_interval,            METH_VARARGS},
	{"set_max_tick",          py_set_max_tick,                 METH_VARARGS},
	{"get_max_tick",          py_get_max_tick,                 METH_VARARGS},
	{"set_elapsed_time",      py_set_elapsed_time,             METH_VARARGS},
	{"get_elapsed_time",      py_get_elapsed_time,             METH_VARARGS},
	{"set_debug_flag1",       py_set_debug_flag1,              METH_VARARGS},
	{"get_debug_flag1",       py_get_debug_flag1,              METH_VARARGS},
	{"set_debug_flag2",       py_set_debug_flag2,              METH_VARARGS},
	{"get_debug_flag2",       py_get_debug_flag2,              METH_VARARGS},
	{"set_B_debug_flag1",     py_set_B_debug_flag1,            METH_VARARGS},
	{"get_B_debug_flag1",     py_get_B_debug_flag1,            METH_VARARGS},
	{"set_B_debug_flag2",     py_set_B_debug_flag2,            METH_VARARGS},
	{"get_B_debug_flag2",     py_get_B_debug_flag2,            METH_VARARGS},
	{"get_sim_voltage",       py_get_sim_voltage,              METH_VARARGS},
	{"get_sim_time",          py_get_sim_time,                 METH_VARARGS},
	{"get_emu_voltage",       py_get_emu_voltage,              METH_VARARGS},
	{"get_emu_time",          py_get_emu_time,                 METH_VARARGS},
	{"get_mea_voltage",       py_get_mea_voltage,              METH_VARARGS},
	{"get_mea_current",       py_get_mea_current,              METH_VARARGS},
	{"get_mea_avg_current",   py_get_mea_avg_current,          METH_VARARGS},
	{"test_emu_voltage",      py_test_emu_voltage,             METH_VARARGS},

	{"set_vref_adc",          py_set_vref_adc,                 METH_VARARGS},
	{"get_vref_adc",          py_get_vref_adc,                 METH_VARARGS},
	{"set_vref_dac",          py_set_vref_dac,                 METH_VARARGS},
	{"get_vref_dac",          py_get_vref_dac,                 METH_VARARGS},
	{"set_rc_adc",            py_set_rc_adc,                   METH_VARARGS},
	{"get_rc_adc",            py_get_rc_adc,                   METH_VARARGS},

	{"set_voffset_adc",       py_set_voffset_adc,              METH_VARARGS},
	{"get_voffset_adc",       py_get_voffset_adc,              METH_VARARGS},
	{"set_voffset_dac",       py_set_voffset_dac,              METH_VARARGS},
	{"get_voffset_dac",       py_get_voffset_dac,              METH_VARARGS},
	{"set_r_vreg",            py_set_r_vreg,                   METH_VARARGS},
	{"get_r_vreg",            py_get_r_vreg,                   METH_VARARGS},
    {"set_coffset_adc",       py_set_coffset_adc,              METH_VARARGS},
    {"get_coffset_adc",       py_get_coffset_adc,              METH_VARARGS},

	{NULL, NULL},
};

static PyObject *B_sharpError;

#ifdef _WIN32
void _declspec(dllexport)
#else
DL_EXPORT(void)
#endif

initB_sharp(void)
{
	PyObject *module, *dict;

	module = Py_InitModule("B_sharp", B_sharpMethods);
	
	build_variable();

	/*
	PyModule_AddObject(module, "device", py_interface);
	PyModule_AddObject(module, "sim_interval", py_sim_interval);
	PyModule_AddObject(module, "tick_interval", py_tick_interval);
	PyModule_AddObject(module, "max_tick", py_max_tick);
	PyModule_AddObject(module, "elapsed_time", py_elapsed_time);


	PyModule_AddObject(module, "debug_flag", py_debug_flag);

	PyModule_AddObject(module, "mea_avg_current", py_mea_avg_current);

	PyModule_AddObject(module, "mea_current", py_mea_current);
	PyModule_AddObject(module, "mea_current_count", py_mea_current_count);

	PyModule_AddObject(module, "mea_voltage", py_mea_voltage);
	PyModule_AddObject(module, "mea_voltage_count", py_mea_voltage_count);

	PyModule_AddObject(module, "sim_voltage", py_sim_voltage);
	PyModule_AddObject(module, "sim_time", py_sim_time);
	PyModule_AddObject(module, "sim_count", py_sim_count);

	PyModule_AddObject(module, "emu_voltage", py_emu_voltage);
	PyModule_AddObject(module, "emu_time", py_emu_time); 
	PyModule_AddObject(module, "emu_voltage_count", py_emu_voltage_count); 
	*/


	dict = PyModule_GetDict(module);

	B_sharpError = PyErr_NewException("B_sharp.error", NULL, NULL);
	PyDict_SetItemString(dict, "error", B_sharpError);

	return;
}
