##	Copyright (c) 2004 The Regents of the University of California. All Rights
##	Reserved. Permission to use, copy, modify, and distribute this  software
##	and its documentation for educational, research and non-profit purposes,
##	without fee, and without a written agreement is hereby granted, provided
##	that the above copyright notice, this paragraph and the following two
##	paragraphs appear in all copies. This software program and documentation
##	are copyrighted  by The Regents of the University of California ("The
##	University of California").
##
##	THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
##	ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
##	UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
##	PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
##	THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
##	TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.
##
##	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
##	DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
##	LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
##	DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
##	THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
##	DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##	WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
##	SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
##	CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
##	ENHANCEMENTS, OR  MODIFICATIONS.

import	os, sys, time
from	B_sharp import *
import	B_file_util

sim_model_suffix = '.model'
sim_model_lst = 'dual.lst'

def	parseArgs(args):
	argc = len(args)	
	if argc == 1 or args[1] == '-h':
		#print 'Usage: %s [emu/pro] [max_time=600] [eth0] [blocking] [sim_interval=0.5] [max_tick=15] [debug1=0] [debug2=1] [B_debug1=0] [B_debug2=0]' % args[0]
		print 'Usage: %s [emu/pro] [max_time=600]' % args[0]
		sys.exit(1)	

	mode = 'emu'
	if argc > 1:
		mode = args[1]

	max_time = 600.	
	if argc > 2:	
		max_time = float(args[2])	

##	if sys.platform == 'win32':
##		os.system('iflist > net.tmp')
##		interface = open('net.tmp').readlines()[0].strip()
##	else:
##		interface = 'eth0'
##
##	if argc > 3:
##		if sys.platform != 'win32':
##			interface = args[3]	
##	set_interface(interface)
##
##	blocking = 1	
##	if argc > 4:	
##		blocking = int(args[4])	
##	set_blocking(blocking)	
##		
##	sim_interval = 0.5
##	if argc > 5:	
##		sim_interval = float(args[5])	
##	set_sim_interval(sim_interval)	
##
##	max_tick = 15 	
##	if argc > 6:	
##		max_tick = int(args[6])	
##	set_max_tick(max_tick)	
##
##	debug_flag1 = 0	
##	if argc > 7:	
##		debug_flag1 = int(args[7])	
##	set_debug_flag1(debug_flag1)	
##
##	debug_flag2 = 0	
##	if argc > 8:	
##		debug_flag2 = int(args[8])	
##	set_debug_flag2(debug_flag2)	
##
##	B_debug_flag1 = 0	
##	if argc > 9:	
##		B_debug_flag1 = int(args[9])	
##	set_B_debug_flag1(B_debug_flag1)	
##
##	B_debug_flag2 = 0	
##	if argc > 10:	
##		B_debug_flag2 = int(args[10])	
##	set_B_debug_flag2(B_debug_flag2)	

	initParameter()
	
	return mode, max_time


def	initParameter():
	init_parameter()

	
def	runProfiling(max_time = 600, n_max = 10, cutoff_voltage = 5.):
	trace_short = B_file_util.get_pro_filename(B_file_util.label_txt)
	trace_long = B_file_util.get_pro_filename(B_file_util.label_txt) + '_long'

	f_short, f_long = initProfiling(trace_short, trace_long)
	
	start_time = time.time()
	last_time = 0
	while last_time <= max_time:
		r = profileNextStep(f_short, f_long, start_time, n_max, last_time)
		if r == None:
			break
		last_time, ca, va = r
		if va < cutoff_voltage:
			break

	completeProfiling(f_short, f_long)
	
	trace2 = 'profiling.pro'
	B_file_util.convertShortProfilingTrace(trace_short, trace2)

	trace2_long = 'profiling_long.pro'
	B_file_util.convertLongProfilingTrace(trace_long, trace2_long)
	
	
def	initProfiling(file_short, file_long):
	init_profiling()

	print 'Profiling started at ', time.ctime()
	print 'Writing raw data trace to file:', file_short, file_long, '...'
	
	fs = open(file_short, 'w')
	#print >> fs, '# start', time.ctime()

	fl = open(file_long, 'w')
	print >> fl, '# start', time.ctime(), 'file = ', file_long

	return fs, fl
	

def	profileNextStep(fs, fl, start_time, n_max, last_time):
	n = 0
	ca = 0
	va = 0
	while n < n_max:
		if run_profiling_step() != 0:
			return None
		c = get_mea_current()
		v = get_mea_voltage()
		ca += c[0]
		va += v[0]
		print >> fl, '\t%12.6f' % c[0], '\t%12.6f' % v[0]
		n += 1
		
	real_time = time.time() - start_time
	time_interval = real_time - last_time
	ca /= n_max
	va /= n_max
	print >> fl, '#\t%12.6f' % ca, '\t%12.6f' % va
	#print >> fl, '!\t%12.6f' % real_time, '\t%12.6f' % time_interval, '\t', n_max, '\t%12.6f' % (n / time_interval)
	print >> fl, '!\t%12.6f' % real_time, '\t%12.6f' % time_interval, '\t', n_max
	
	print >> fs, '\t%12.6f' % real_time, '\t%12.6f' % ca, '\t%12.6f' % va

	return real_time, ca, va


def	completeProfiling(fs, fl):
	complete_profiling()
	#print >> fs, '# finish', time.ctime()
	fs.close()

	print >> fl, '# finish', time.ctime()
	fl.close()

	print 'Profiling finished at ', time.ctime()


def	runEmulation(sim_model, max_time = 600):
	trace1 = B_file_util.get_emu_filename(B_file_util.label_txt)

	f = initEmulation(sim_model, trace1)

	start_time = time.time()
	elapsed_time = 0

	points = 0
	while elapsed_time <= max_time:
		points += 1
		r = emulateNextStep(f, start_time, points)
		if r == None:
			break
		elapsed_time, c, t, v = r
		
	completeEmulation(f)
	
	trace2 = 'emulation.emu'
	B_file_util.convertEmulationTrace(trace1, trace2)

	return


def	initEmulation(sim_model, file = None):
	setSimModel(sim_model)
	
	init_emulation()

	if file == None:
		return None
	
	print 'Emulation started at ', time.ctime()
	print 'Writing raw data trace to file:', file, '...'
	
	f = open(file, 'w')
	print >> f, '# start', time.ctime(), 'file =', file

	return f
	

def	emulateNextStep(f, start_time, count = 1):
	c = get_mea_avg_current()
	if run_emulation_step() != 0:
		return None
	elapsed_time = get_elapsed_time()

	#real_time = time.time() - start_time
	#time_drift = real_time - elapsed_time

	t, v = get_emu_time(), get_emu_voltage()
	for i in range(len(t)):
		print >> f, '\t%12.6f' % t[i], '\t%12.6f' % c, '\t%12.6f' % v[i]

	
	while 1:
		real_time = time.time() - start_time
		time_drift = real_time - elapsed_time
		if time_drift < -0.002:
			time.sleep(0.001)
		else:
			break

	print >> f, '#\t%12.6f' % real_time, '\t%12.6f' % elapsed_time, '\t%12.6f' % time_drift, '\t%12.6f' % (time_drift / count)

	return elapsed_time, c, t, v


def	completeEmulation(f):
	complete_emulation()

	print >> f, '# finish', time.ctime()
	f.close()

	print 'Emulation finished at ', time.ctime()




# just paste from dualfoil_util.py

def	setSimModel(sim_model):
	print 'Using simulation model:', sim_model
	
	sim_cfg_lines = open(sim_model + sim_model_suffix).readlines()
	f = open(sim_model_lst, 'w')
	for line in sim_cfg_lines:
		print >> f, line.strip()
	f.close()


def	runSimulation(t, c, sim_model, max_time = 600):
	trace1 = B_file_util.get_sim_filename(B_file_util.label_txt)

	f = initSimulation(sim_model, trace1)

#	start_time = time.time()
#	elapsed_time = 0
	
	for i in range(len(t) - 1, 0, -1):
		t[i] -= t[i - 1]

	for i in range(len(t)):
		r = simulateNextStep(f, t[i], c[i])
		if r == None:
			break
		elapsed_time = r[0][-1]
		if elapsed_time >= max_time:
			break
		
	completeSimulation(f)
	
	trace2 = 'simulation.sim'
	B_file_util.convertSimulationTrace(trace1, trace2)

	return


def	initSimulation(sim_model, file):
	setSimModel(sim_model)
	
	init_simulation()

	print 'Simulation started at ', time.ctime()
	print 'Writing raw data trace to file:', file, '...'
		
	f = open(file, 'w')
	print >> f, '# start', time.ctime(), 'file = ', file

	return f
	

def	simulateNextStep(f, ti, c):
	r = simulate_next_step(c, ti, 1)
	t = get_sim_time()
	v = get_sim_voltage()

	for i in range(len(t)):
		print >> f, '\t%12.6f' % t[i], '\t%12.6f' % c, '\t%12.6f' % v[i]

	return r, t, v
	

def	completeSimulation(f, r):
	complete_simulation(r)

	print >> f, '# finish', time.ctime()
	f.close()

	print 'Simulation finished at ', time.ctime()


