##	Copyright (c) 2004 The Regents of the University of California. All Rights
##	Reserved. Permission to use, copy, modify, and distribute this  software
##	and its documentation for educational, research and non-profit purposes,
##	without fee, and without a written agreement is hereby granted, provided
##	that the above copyright notice, this paragraph and the following two
##	paragraphs appear in all copies. This software program and documentation
##	are copyrighted  by The Regents of the University of California ("The
##	University of California").
##
##	THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
##	ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
##	UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
##	PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
##	THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
##	TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.
##
##	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
##	DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
##	LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
##	DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
##	THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
##	DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##	WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
##	SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
##	CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
##	ENHANCEMENTS, OR  MODIFICATIONS.

import	sys, time
from	py_dualfoil import *
import	B_file_util

# must make changes to B_sharp_util too

sim_model_suffix = '.model'
sim_model_lst = 'dual.lst'


def	setSimModel(sim_model):
	print 'Using simulation model:', sim_model
	
	sim_cfg_lines = open(sim_model + sim_model_suffix).readlines()
	f = open(sim_model_lst, 'w')
	for line in sim_cfg_lines:
		print >> f, line.strip()
	f.close()


def	runSimulation(t, c, sim_model, max_time = 600):
	trace1 = B_file_util.get_sim_filename(B_file_util.label_txt)

	f = initSimulation(sim_model, trace1)

#	start_time = time.time()
#	elapsed_time = 0
	
	for i in range(len(t) - 1, 0, -1):
		t[i] -= t[i - 1]

	for i in range(len(t)):
		r = simulateNextStep(f, t[i], c[i])
		if r == None:
			break
		elapsed_time = r[0][-1]
		if elapsed_time >= max_time:
			break
		
	completeSimulation(f)
	
	trace2 = 'simulation.sim'
	B_file_util.convertSimulationTrace(trace1, trace2)

	return


def	initSimulation(sim_model, file):
	setSimModel(sim_model)

	init_simulation()
	
	print 'Simulation started at ', time.ctime()
	print 'Writing raw data trace to file:', file, '...'
	
	f = open(file, 'w')
	print >> f, '# start', time.ctime(), 'file = ', file

	return f
	

def	simulateNextStep(f, ti, c):
	r = simulate_next_step(c, ti, 1)
	t = get_sim_time()
	v = get_sim_voltage()

	for i in range(len(t)):
		print >> f, '\t%12.6f' % t[i], '\t%12.6f' % c, '\t%12.6f' % v[i]

	return r, t, v
	

def	completeSimulation(f, r):
	complete_simulation(r)

	print >> f, '# finish', time.ctime()
	f.close()

	print 'Simulation finished at ', time.ctime()

