#!/usr/bin/env python

##	Copyright (c) 2004 The Regents of the University of California. All Rights
##	Reserved. Permission to use, copy, modify, and distribute this  software
##	and its documentation for educational, research and non-profit purposes,
##	without fee, and without a written agreement is hereby granted, provided
##	that the above copyright notice, this paragraph and the following two
##	paragraphs appear in all copies. This software program and documentation
##	are copyrighted  by The Regents of the University of California ("The
##	University of California").
##
##	THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
##	ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
##	UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
##	PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
##	THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
##	TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.
##
##	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
##	DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
##	LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
##	DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
##	THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
##	DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##	WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
##	SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
##	CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
##	ENHANCEMENTS, OR  MODIFICATIONS.

from	Tkinter import *
import	tkFileDialog

import	sys, os, time
#import	thread

import	sframe
import	net_dlg, pro_dlg, emu_dlg, sim_dlg, clb_dlg, dbg_dlg, cmd_dlg, scl_dlg, stt_dlg, pnt_dlg, abt_dlg
import	my_tkBusyMan
import	B_file_util


class	UI(Frame):
	def __init__(self, master, demo_mode):
		Frame.__init__(self, master, bd = 2, relief = RAISED)
		self.master = master

		self.demo_mode = demo_mode

		self.busy_man = my_tkBusyMan.BusyManager(self)

		self.initParameter()
		self.initMenu()
		self.initFrame()

		return


	def	initParameter(self):
		if self.demo_mode:
			self.util = __import__('dualfoil_util')
		else:
			self.util = __import__('B_sharp_util')

		self.options = [pnt_dlg, scl_dlg, net_dlg, pro_dlg, emu_dlg, sim_dlg, clb_dlg, dbg_dlg, cmd_dlg]
		
		self.loadOptions()

		if demo_mode != 1:
			self.util.initParameter()

		self.pro_data_file = ''
		self.emu_data_file = ''
		self.sim_data_file = ''
		self.pos_data_file = ''

		self.pro_trace_file = ''
		self.emu_trace_file = ''
		self.sim_trace_file = ''
		
			
	def	initFrame(self):

		self.pro_curve_index = 0
		self.emu_curve_index = 1
		self.sim_curve_index = 2
		self.pos_curve_index = 3

		self.v_chart_size = self.c_chart_size = (750, 250)

		color_list = [None] * 4 
		color_list[self.pro_curve_index] = self.pro_color
		color_list[self.emu_curve_index] = self.emu_color
		color_list[self.sim_curve_index] = self.sim_color
		color_list[self.pos_curve_index] = self.pos_color
		
		width_list = [None] * 4
		width_list[self.pro_curve_index] = self.pro_width
		width_list[self.emu_curve_index] = self.emu_width
		width_list[self.sim_curve_index] = self.sim_width
		width_list[self.pos_curve_index] = self.pos_width


		self.v_frame, self.v_chart = sframe.sframe(self,
												   'Voltage',
												   self.v_chart_size, 
												   (self.t_left, self.v_bottom, self.t_right, self.v_top),
												   (self.t_min, self.v_min, self.t_max, self.v_max),
												   4,
												   color_list,
												   width_list
												   )


		self.t_left = self.v_chart.x_left
		self.t_right = self.v_chart.x_right
		self.t_min = self.v_chart.x_min
		self.t_max = self.v_chart.x_max

		self.v_bottom = self.v_chart.y_bottom
		self.v_top = self.v_chart.y_top
		self.v_min = self.v_chart.y_min
		self.v_max = self.v_chart.y_max

		self.c_frame, self.c_chart = sframe.sframe(self,
												   'Current',
												   self.c_chart_size, 
												   (self.t_left, self.c_bottom, self.t_right, self.c_top),
												   (self.t_min, self.c_min, self.t_max, self.c_max),
												   4,
												   color_list,
												   width_list
												   )

		self.v_chart.set_sync_sibling([self.c_chart], [])
		self.c_chart.set_sync_sibling([self.v_chart], [])

		self.t_left = self.c_chart.x_left
		self.t_right = self.c_chart.x_right
		self.t_min = self.c_chart.x_min
		self.t_max = self.c_chart.x_max

		self.c_bottom = self.c_chart.y_bottom
		self.c_top = self.c_chart.y_top
		self.c_min = self.c_chart.y_min
		self.c_max = self.c_chart.y_max

		self.p_frame = Frame(self, bd = 2, relief = RAISED)

		cmd_panel = Frame(self.p_frame)
		
		self.p_run_profiling = Button(cmd_panel, text = 'Profiling', command = self.runProfiling)
		self.p_run_profiling.pack(side = LEFT, padx = 10, pady = 5)
		
		self.p_run_emulation = Button(cmd_panel, text = 'Emulation', command = self.runEmulation)
		self.p_run_emulation.pack(side = LEFT, padx = 10, pady = 5)
	
		self.p_run_simulation = Button(cmd_panel, text = 'Simulation', command = self.runSimulation)
		self.p_run_simulation.pack(side = LEFT, padx = 10, pady = 5)
		
		self.p_run_stop = Button(cmd_panel, text = 'Stop', command = self.runStop)
		self.p_run_stop.pack(side = LEFT, padx = 10, pady = 5)

		cmd_panel.pack()
		
		self.p_frame.pack(side = TOP, anchor = N, fill = X, expand = 1)

		return


	def	initMenu(self):
		menu = Menu(self)
		
		self.master.config(menu = menu)

		self.menu = menu

		filemenu = Menu(menu)
		menu.add_cascade(label = 'File', menu = filemenu)
		filemenu.add_command(label = 'Open Profiling ...', command = self.openProfiling)
		filemenu.add_command(label = 'Save Profiling ...', command = self.saveProfiling)
		filemenu.add_separator()
		filemenu.add_command(label = 'Open Emulation ...', command = self.openEmulation)
		filemenu.add_command(label = 'Save Emulation ...', command = self.saveEmulation)
		filemenu.add_separator()
		filemenu.add_command(label = 'Open Simulation ...', command = self.openSimulation)
		filemenu.add_command(label = 'Save Simulation ...', command = self.saveSimulation)
		filemenu.add_separator()
		filemenu.add_command(label = 'Open Post-profiling ...', command = self.openPostprofiling)
		filemenu.add_separator()
		filemenu.add_command(label = 'Statistics ...', command = self.openSttDlg)
		filemenu.add_separator()
		filemenu.add_command(label = 'Exit', command = self.quit)
		
		runmenu = Menu(menu)
		menu.add_cascade(label = 'Run', menu = runmenu)
		runmenu.add_command(label = 'Profiling', command = self.runProfiling)
		runmenu.add_command(label = 'Emulation', command = self.runEmulation)
		runmenu.add_command(label = 'Simulation', command = self. runSimulation)
		runmenu.add_separator()	
		runmenu.add_command(label = 'Stop', command = self.runStop)

		optmenu = Menu(menu)
		menu.add_cascade(label = 'Options', menu = optmenu)
		optmenu.add_command(label = 'Scale ...', command = self.openSclDlg)
		optmenu.add_command(label = 'Paint ...', command = self.openPntDlg)
		optmenu.add_separator()
		optmenu.add_command(label = 'Network ...', command = self.openNetDlg)
		optmenu.add_separator()
		optmenu.add_command(label = 'Profiling ...', command = self.openProDlg)
		optmenu.add_command(label = 'Emulation ...', command = self.openEmuDlg)
		optmenu.add_command(label = 'Simulation ...', command = self.openSimDlg)
		optmenu.add_separator()
		optmenu.add_command(label = 'Calibration ...', command = self.openClbDlg)
		optmenu.add_separator()
		optmenu.add_command(label = 'Debug Flag ...', command = self.openDbgDlg)
		optmenu.add_command(label = 'Debug Cmd ...', command = self.openCmdDlg)
		optmenu.add_separator()
		optmenu.add_command(label = 'Reset All', command = self.resetOptions)
		optmenu.add_command(label = 'Save All', command = self.saveOptions)
		
		helpmenu = Menu(menu)
		menu.add_cascade(label = 'Help', menu = helpmenu)
		helpmenu.add_command(label = 'About ...', command = self.openAbtDlg)

		return

		
	def	quit(self):
		sys.exit(0)


	def	openPostprofiling(self):
		file = self.openFile('Open Post-profiling Data', '.pro', ('Post-profiling Data', '.pro'))
		if file != '':
			self.showPostprofilingFile(file)
			self.pos_data_file = file


	def	openProfiling(self):
		file = self.openFile('Open Profiling Data', '.pro', ('Profiling Data', '.pro'))
		if file != '':
			self.showProfilingFile(file)
			self.pro_trace_file = ''
			self.pro_data_file = file

		
	def	saveProfiling(self):
		if not self.demo_mode:
			if self.pro_trace_file == '':
				print 'No profiling data'
				return

		file = self.saveFile('Save Profiling Data', '.pro', ('Profiling Data', '.pro'))
		if file != '':
			self.saveProfilingFile(file)
			self.pro_data_file = file
		
		
	def	openEmulation(self):
		file = self.openFile('Open Emulation Data', '.emu', ('Emulation Data', '.emu'))
		if file != '':
			self.showEmulationFile(file)
			self.emu_trace_file = ''
			self.emu_data_file = file


	def	saveEmulation(self):
		if not self.demo_mode:
			if self.emu_trace_file == '':
				print 'No emulation data'
				return
		
		file = self.saveFile('Save Emulation Data', '.emu', ('Emulation Data', '.emu'))
		if file != '':
			self.saveEmulationFile(file)
			self.emu_data_file = file
		

	def	openSimulation(self):
		file = self.openFile('Open Simulation Data', '.sim', ('Simulation Data', '.sim'))
		if file != '':
			self.showSimulationFile(file)
			self.sim_trace_file = ''
			self.sim_data_file = file


	def	saveSimulation(self):
		if not self.demo_mode:
			if self.sim_trace_file == '':
				print 'No simulation data'
				return

		file = self.saveFile('Save Simulation Data', '.sim', ('Simulation Data', '.sim'))
		if file != '':
			self.saveSimulationFile(file)
			self.sim_data_file = file
		

	def	openFile(self, title, ext, ext_description):
		file = tkFileDialog.askopenfilename(title = title,
								defaultextension = ext,
								filetypes = (
									ext_description, 
									('Text Files', '.txt'),
									('All Files', '*'))
							   )
		return file
	

	def	saveFile(self, title, ext, ext_description):
		file = tkFileDialog.asksaveasfilename(title = title,
									defaultextension = ext,
									filetypes = (
										ext_description, 
										('Text Files', '.txt'),
										('All Files', '*'))
				)
		return file


	def	do_test(self, curve_index):
		import random
		points = 0

		start_time = time.time()
		while not self.stop:
			ca = 0
			va = 0
			n = 0
			while n < self.pro_sample_count:
				ca += random.uniform(0, 0.5)
				va += random.uniform(0, 1)
				n += 1
			ca /= self.pro_sample_count
			va /= self.pro_sample_count
			ca *= random.uniform(0, 1)
			va *= random.uniform(0, 1)
			va += 3.5
			
			t = time.time() - start_time
			t *= 10
			if t > self.pro_time:
				break
			if points == 0:
				self.v_chart.updateData([0, t], [va, va], curve_index)
				self.c_chart.updateData([0, t], [ca, ca], curve_index)
			else:
				self.v_chart.appendData([t], [va], curve_index)
				self.c_chart.appendData([t], [ca], curve_index)
			points += 1


	def	runProfiling(self):
		if self.demo_mode:
			self.openProfiling()
			return
		
		self.v_chart.clearChart(self.pro_curve_index)
		self.v_chart.drawLabel('Profiling: Voltage', self.pro_curve_index)
		self.v_chart.rollBack()

		self.c_chart.clearChart(self.pro_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Profiling: Current', self.pro_curve_index)

		self.stop = 0
		if self.pro_time > self.t_max:
			self.pro_time = self.t_max

		self.setWait([self.p_run_stop])

		#thread.start_new(self.do_profiling, ())
		self.do_profiling()
		return


	def	do_profiling(self):
		self.pro_data_file = ''
		self.pro_trace_file = B_file_util.get_pro_filename(B_file_util.label_gui)
		self.pro_trace_file_long = self.pro_trace_file + '_long'
		fs, fl = self.util.initProfiling(self.pro_trace_file, self.pro_trace_file_long)

		start_time = time.time()
		last_time = 0
		points = 0

		while not self.stop:
			r = self.util.profileNextStep(fs, fl, start_time, self.pro_sample_count, last_time)
			if r == None:
				break
			last_time, ca, va = r
			if last_time > self.pro_time:
				break
			if points == 0:
				self.v_chart.updateData([0, last_time], [va, va], self.pro_curve_index)
				self.c_chart.updateData([0, last_time], [ca, ca], self.pro_curve_index)
			else:
				self.v_chart.appendData([last_time], [va], self.pro_curve_index)
				self.c_chart.appendData([last_time], [ca], self.pro_curve_index)
			points += 1

			self.update()
			#time.sleep(0.02)
			
			if va < self.pro_cutoff_voltage:
				break

		self.util.completeProfiling(fs, fl)

		self.clearWait()
		self.stop = 1


	def	runEmulation(self):
		if self.demo_mode:
			self.openEmulation()
			return

		self.v_chart.clearChart(self.emu_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Emulation: Voltage', self.emu_curve_index)
		
		self.c_chart.clearChart(self.emu_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Emulation: Current', self.emu_curve_index)
		
		self.stop = 0
		if self.emu_time > self.t_max:
			self.emu_time = self.t_max

		self.setWait([self.p_run_stop])
		
		#thread.start_new(self.do_emulation, ())
		self.do_emulation()
		return


	def	do_emulation(self):
		self.emu_data_file = ''
		self.emu_trace_file = B_file_util.get_emu_filename(B_file_util.label_gui)
		f = self.util.initEmulation(self.sim_model, self.emu_trace_file)

		start_time = time.time()
		points = 0

		while not self.stop:
			points += 1
			r = self.util.emulateNextStep(f, start_time, points)
			if r == None:
				break
			elapsed_time, ca, t, v = r
			if elapsed_time > self.emu_time:
				break
			if points == 1:
				self.v_chart.updateData(t, v, self.emu_curve_index)
				self.c_chart.updateData([0, elapsed_time], [ca, ca], self.emu_curve_index)
			else:
				self.v_chart.appendData(t, v, self.emu_curve_index)
				self.c_chart.appendData([elapsed_time], [ca], self.emu_curve_index)

			self.update()
			#time.sleep(0.02)

		self.util.completeEmulation(f)
		
		self.clearWait()
		self.stop = 1
		

	def	runSimulation(self):
		n = len(self.c_chart.x_data[self.pro_curve_index])
		if n == 0:
			print 'No profiling data'
			return
			
		self.v_chart.clearChart(self.sim_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Simulation: Voltage', self.sim_curve_index)
		
		self.c_chart.clearChart(self.sim_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Simulation: Current', self.sim_curve_index)
		
		self.stop = 0
		if self.sim_time > self.t_max:
			self.sim_time = self.t_max

		self.setWait([self.p_run_stop])
		
		#thread.start_new(self.do_simulation, ())
		self.do_simulation()
		return


	def	do_simulation(self):
		self.sim_data_file = ''
		self.sim_trace_file = B_file_util.get_sim_filename(B_file_util.label_gui)
		f = self.util.initSimulation(self.sim_model, self.sim_trace_file)

		elapsed_time = 0.
		i = 0
		n = len(self.c_chart.x_data[self.pro_curve_index])
		dualfoil_sim_interval = self.sim_interval / 60.
		ca_last = 0
		iter = 0

		while i < n:
			ca = 0
			na = 0
			next_time = elapsed_time + self.sim_interval

			while self.c_chart.x_data[self.pro_curve_index][i] < next_time:
				ca += self.c_chart.y_data[self.pro_curve_index][i]
				i += 1
				na += 1
				if i >= n:
					break
			if na != 0:
				ca /= na
			else:
				ca = ca_last
			ca_last = ca

			if next_time > self.sim_time:
				break
			
			r = self.util.simulateNextStep(f, dualfoil_sim_interval, ca)
			
			r, t, v = r
			
			next_time = t[-1]
			
			if iter == 0:
				self.v_chart.updateData(t, v, self.sim_curve_index)
				self.c_chart.updateData([elapsed_time, next_time], [ca, ca], self.sim_curve_index)
			else:
				self.v_chart.appendData(t, v, self.sim_curve_index)
				self.c_chart.appendData([elapsed_time, next_time], [ca, ca], self.sim_curve_index)

			self.update()
			#time.sleep(0.02 * len(v))

			elapsed_time = next_time
			iter += 1

			if self.stop:
				break
			
		self.util.completeSimulation(f, r)

		self.clearWait()
		self.stop = 0
			

	def	runStop(self):
		self.stop = 1
		return


	def	setWait(self, active_items = []):
		l = []
		self.busy_man.busy(self.master, l + active_items)
		self.update()


	def	clearWait(self):
		self.busy_man.notbusy()
		self.update()

		
	def	showPostprofilingFile(self, file):
		self.setWait()

		t, c, v, e, ic, iv = B_file_util.parseTraceFile(file)

		self.v_chart.clearChart(self.pos_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Post-profiling: Voltage', self.pos_curve_index)
		self.v_chart.updateData(t, v, self.pos_curve_index)

		self.c_chart.clearChart(self.pos_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Post-profiling: Current', self.pos_curve_index)
		self.c_chart.updateData(t, c, self.pos_curve_index)

		#self.pro_data_file = file
		#self.pro_trace_file = ''

		self.clearWait()


	def	showProfilingFile(self, file):
		self.setWait()

		t, c, v, e, ic, iv = B_file_util.parseTraceFile(file)
		
		self.v_chart.clearChart(self.pro_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Profiling: Voltage', self.pro_curve_index)
		self.v_chart.updateData(t, v, self.pro_curve_index)

		self.c_chart.clearChart(self.pro_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Profiling: Current', self.pro_curve_index)
		self.c_chart.updateData(t, c, self.pro_curve_index)

		#self.pro_data_file = file
		#self.pro_trace_file = ''
		
		self.clearWait()


	def	showEmulationFile(self, file):
		self.setWait()

		t, c, v, e, ic, iv = B_file_util.parseTraceFile(file)
		
		self.v_chart.clearChart(self.emu_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Emulation: Voltage', self.emu_curve_index)
		self.v_chart.updateData(t, v, self.emu_curve_index)
		
		self.c_chart.clearChart(self.emu_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Emulation: Current', self.emu_curve_index)
		self.c_chart.updateData(t, c, self.emu_curve_index)

		#self.emu_data_file = file
		#self.emu_trace_file = ''

		self.clearWait()


	def	showSimulationFile(self, file):
		self.setWait()

		t, c, v, e, ic, iv = B_file_util.parseTraceFile(file)
		
		self.v_chart.clearChart(self.sim_curve_index)
		self.v_chart.rollBack()
		self.v_chart.drawLabel('Simulation: Voltage', self. sim_curve_index)
		self.v_chart.updateData(t, v, self. sim_curve_index)
		
		self.c_chart.clearChart(self. sim_curve_index)
		self.c_chart.rollBack()
		self.c_chart.drawLabel('Simulation: Current', self. sim_curve_index)
		self.c_chart.updateData(t, c, self. sim_curve_index)

		#self.sim_data_file = file
		#self.sim_trace_file = ''

		self.clearWait()


	def	saveProfilingFile(self, file):
		if self.pro_trace_file == '':
			print 'No profiling data'
			return
		
		self.setWait()

		B_file_util.convertShortProfilingTrace(self.pro_trace_file, file)

		i = file.find('.pro')
		file_long = file[:i] + '_long' + file[i:]
		B_file_util.convertLongProfilingTrace(self.pro_trace_file_long, file_long)
		
		#self.pro_data_file = file

		self.clearWait()

		
	def	saveEmulationFile(self, file):
		if self.emu_trace_file == '':
			print 'No emulation data'
			return
		
		self.setWait()

		B_file_util.convertEmulationTrace(self.emu_trace_file, file)
		#self.emu_data_file = file

		self.clearWait()


	def	saveSimulationFile(self, file):
		if self.sim_trace_file == '':
			print 'No simulation data'
			return
		
		self.setWait()

		B_file_util.convertSimulationTrace(self.sim_trace_file, file)
		#self.sim_data_file = file

		self.clearWait()

		
	def	openSclDlg(self, event = None):
		scl_dlg.scl_dlg(self)
		

	def openPntDlg(self, event = None):
		pnt_dlg.pnt_dlg(self)


	def	openNetDlg(self, event = None):
		net_dlg.net_dlg(self)


	def	openProDlg(self, event = None):
		pro_dlg.pro_dlg(self)


	def	openEmuDlg(self, event = None):
		emu_dlg.emu_dlg(self)


	def	openSimDlg(self, event = None):
		sim_dlg.sim_dlg(self)


	def	openClbDlg(self, event = None):
		clb_dlg.clb_dlg(self)


	def	openDbgDlg(self, event = None):
		dbg_dlg.dbg_dlg(self)
			

	def openCmdDlg(self, event = None):
		cmd_dlg.cmd_dlg(self)


	def	openAbtDlg(self, event = None):
		abt_dlg.abt_dlg(self)
		
	def	saveOptions(self, event = None):
		for o in self.options:
			o.write(self)
	

	def	loadOptions(self, event = None):
		for o in self.options:
			o.read(self)
		

	def	resetOptions(self, event = None):
		for o in self.options:
			try:
				os.remove(o.cfg)
			except:
				pass
		self.loadOptions()

		
	def	openSttDlg(self, event = None):
		label1 = ['Energy', 'V(start)', 'V(end)']
		par1 = ['e', 'v0', 'v1']
		format1 = ['%8.3f', '%8.3f', '%8.3f']
		label2 = ['Diff.(Energy)', 'Diff.(|V|)', 'Diff.(|C|)']
		par2 = ['err_e', 'err_v_abs', 'err_c_abs']
		format2 = ['%6.3f%%', '%6.3f%%', '%6.3f%%']
		stt_dlg.stt_dlg(self, label1, par1, format1, label2, par2, format2)


	def	extractDataParameter(self, file, pars):
		if file == '':
			print 'Empty filename, cannot extract data'
			return
		vars = B_file_util.extractDataParameter(file, pars)
		if vars.count(None) != 0:
			B_file_util.rebuildDataInfo(file)
			vars = B_file_util.extractDataParameter(file, pars)
		return vars
			
			
	def	extractComparedParameter(self, file, pars):
		if file == '':
			print 'Empty filename, cannot compare'
			return
		if self.pro_data_file == '':
			print 'No profiling data, cannot compare'
			return
		vars = B_file_util.extractComparedParameter(file, self.pro_data_file, pars)
		if vars.count(None) != 0:
			B_file_util.compareTraceFile(file, self.pro_data_file)
			vars = B_file_util.extractComparedParameter(file, self.pro_data_file, pars)
		return vars
		

if __name__ == '__main__':
	if len(sys.argv) == 2 and sys.argv[1] == 'demo':
		demo_mode = 1
	else:
		demo_mode = 0

	root = Tk()
	UI(root, demo_mode).pack()
	title = 'B# - Battery Emulator'

	if demo_mode == 1:
		title += '  (Demo)'

	root.title(title)
	root.mainloop()
