#!/usr/bin/python

import sys, os, threading, getopt, thread, time

import serial, enhancedserial

EXITCHARCTER = '\x04'   #ctrl+d
SENDCHARCTER = '\x01'	#ctrl+a

if os.name == 'nt':
	import msvcrt

	def getkey():
		while 1:
			if echo:
				z = msvcrt.getche()
			else:
				z = msvcrt.getch()
			if z == '\0' or z == '\xe0':    #functions keys
				msvcrt.getch()
			else:
				return z

	def	get_string():
		sys.stderr.write('Enter filename: ')
		file = raw_input()
		if not os.path.exists(file):
			sys.stderr.write('File not found: ' + file + '\n')
			return None
		else:
			f = open(file, 'rb')
			st = f.read()
			f.close()
			return st
		
	def	send_string(s, st):
		for i in range(len(st)):
			ch = st[i]
			s.write(ch)
			while not s.getDSR():
				time.sleep(0.001)

	def	setup_console():
		pass


elif os.name == 'posix':
	import termios, sys, os

	fd = sys.stdin.fileno()
	old = termios.tcgetattr(fd)
	new = termios.tcgetattr(fd)

	def getkey():
		c = os.read(fd, 1)
		if echo: sys.stderr.write(c)
		return c

	def	get_string():
		termios.tcsetattr(fd, termios.TCSAFLUSH, old)
		sys.stderr.write('Enter filename: ')
		file = raw_input()
		termios.tcsetattr(fd, termios.TCSANOW, new)
		if not os.path.exists(file):
			sys.stderr.write('File not found: ' + file + '\n')
			return None
		else:
			f = open(file, 'rb')
			st = f.read()
			f.close()
			return st

	def	send_string(s, st):
		for i in range(len(st)):
			ch = st[i]
			s.write(ch)
			if s.getCTS():
				time.sleep(0.001)
			#time.sleep(0.001)

	def clenaup_console():
		termios.tcsetattr(fd, termios.TCSAFLUSH, old)

	def	setup_console():
		new[3] = new[3] & ~termios.ICANON & ~termios.ECHO
		new[6][termios.VMIN] = 1
		new[6][termios.VTIME] = 0
		termios.tcsetattr(fd, termios.TCSANOW, new)
		
	sys.exitfunc = clenaup_console  #terminal modes have to be restored on exit...

else:
	raise "Sorry no implementation for your platform (%s) available." % sys.platform


			
def reader(s):
	"""loop forever and copy serial->console"""
	while 1:
		st = s.read()
		sys.stderr.write(st)

def writer(s):
	"""loop and copy console->serial until EOF character is found"""
	while 1:
		c = getkey()
		if c == EXITCHARCTER:
			break   #exit on esc
		if c == SENDCHARCTER:
			st = get_string()
			if st != None:
				send_string(s, st)
		else:
			s.write(c)              #send character
		if convert_outgoing_cr and c == '\r':
			s.write('\n')
			if echo: sys.stderr.write('\n')

def usage():
	print >>sys.stderr, """USAGE: %s [options]
	Simple Terminal Programm for the serial port.

	options:
	-p, --port=PORT: port, a number, defualt = 0 or a device name
	-b, --baud=BAUD: baudrate, default 9600
	-r, --rtscts:    enable RTS/CTS flow control (default off)
	-x, --xonxoff:   enable software flow control (default off)
	-e, --echo:      enable local echo (default off)
	-c, --cr:        enable CR -> CR+LF translation (default off)

	""" % sys.argv[0]

if __name__ == '__main__':

	setup_console()
	
	try:
		opts, args = getopt.getopt(sys.argv[1:],
				"hp:b:rxec",
				["help", "port=", "baud=", "rtscts", "xonxoff", "echo", "cr"])
	except getopt.GetoptError:
		# print help information and exit:
		usage()
		sys.exit(2)
	
	port  = 0
	baudrate = 9600
	echo = 0
	convert_outgoing_cr = 0
	rtscts = 0
	xonxoff = 0
	
	for o, a in opts:
		if o in ("-h", "--help"):   #help text
			usage()
			sys.exit()
		elif o in ("-p", "--port"):   #specified port
			try:
				port = int(a)
			except ValueError:
				port = a
		elif o in ("-b", "--baud"):   #specified baudrate
			try:
				baudrate = int(a)
			except ValueError:
				raise ValueError, "Baudrate must be a integer number"
		elif o in ("-r", "--rtscts"):
			rtscts = 1
		elif o in ("-x", "--xonxoff"):
			xonxoff = 1
		elif o in ("-e", "--echo"):
			echo = 1
		elif o in ("-c", "--cr"):
			convert_outgoing_cr = 1

	try:
		s = serial.Serial(port, baudrate, rtscts=rtscts, xonxoff=xonxoff)
	except:
		print "could not open port"
		sys.exit(1)
	print "--- Miniterm --- type Ctrl-D to quit"

	#start serial->console thread
	r = threading.Thread(target=reader, args = (s, ))
	r.setDaemon(1)
	r.start()
	
#	thread.start_new(reader, (s,))

	#enter console->serial loop
	writer(s)

	s.close()	

	print "\n--- exit ---"

