/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include <string.h>
#include <Python.h>

#include "B_dualfoil.h"


#define	RETURN_NONE	\
	{Py_INCREF(Py_None);	return Py_None;}


#define	RETURN_STR(py_obj, value)		\
	return Py_BuildValue("s#", value, strlen(value));

#define	RETURN_INT(value)	\
	return Py_BuildValue("i", value);

#define	RETURN_INT2(value1, value2)	\
	return Py_BuildValue("(ii)", value1, value2);


#define	RETURN_FLT(value)	\
	return Py_BuildValue("d", value);

#define	RETURN_FLT2(value1, value2)	\
	return Py_BuildValue("(dd)", value1, value2);


//#define	PARSE_NONE(args)		\
//	PyArg_ParseTuple(args, "|");


#define	PARSE_STR(args, value)		\
	if (PyArg_ParseTuple(args, "s", &value) == 0)	\
		RETURN_NONE	

#define	PARSE_INT(args, value)		\
	if (PyArg_ParseTuple(args, "i", &value) == 0)	\
		RETURN_NONE	


#define	PARSE_INT2(args, value1, value2)		\
	if (PyArg_ParseTuple(args, "ii", &value1, &value2) == 0)	\
		RETURN_NONE

#define	PARSE_INT3(args, value1, value2, value3)		\
	if (PyArg_ParseTuple(args, "iii", &value1, &value2, &value3) == 0)	\
		RETURN_NONE


#define	PARSE_FLT(args, value)		\
	if (PyArg_ParseTuple(args, "d", &value) == 0)	\
		RETURN_NONE

#define	PARSE_FLT2(args, value1, value2)	\
	if (PyArg_ParseTuple(args, "dd", &value1, &value2) == 0)	\
		RETURN_NONE


#define	PARSE_FLT_INT(args, value1, value2)		\
	if (PyArg_ParseTuple(args, "di", &value1, &value2) == 0)	\
		RETURN_NONE

#define	PARSE_FLT2_INT(args, value1, value2, value3)		\
	if (PyArg_ParseTuple(args, "ddi", &value1, &value2, &value3) == 0)	\
		RETURN_NONE

#define	BUILD_INT(py_obj, value)		\
	py_obj = Py_BuildValue("i", value);

#define	BUILD_FLT(py_obj, value)		\
	py_obj = Py_BuildValue("d", value);

#define	BUILD_STR(py_obj, value)		\
	py_obj = Py_BuildValue("s#", value, strlen(value));


extern	double	sim_voltage[];
extern	double	sim_time[];
extern	int	sim_count;

static	PyObject	*py_sim_voltage;
static	PyObject	*py_sim_time;
static	PyObject	*py_sim_count;


static  PyObject * build_py_flt_array(PyObject **py_array, double *c_array, int array_len)
{
	int	i;
	PyObject *temp;

	//PyObject_Del(*py_array);

	*py_array = PyList_New(array_len);
	//Py_INCREF(*py_array);
	for (i = 0; i < array_len; i ++)
	{
		BUILD_FLT(temp, c_array[i])
		PyList_SetItem(*py_array, i, temp);
		//Py_INCREF(temp);
	}

	RETURN_NONE
}



static PyObject	* py_get_sim_voltage(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_sim_voltage);
	return py_sim_voltage;
}


static PyObject	* py_get_sim_time(PyObject *self, PyObject *args)
{
	//PARSE_NONE(args)
	Py_INCREF(py_sim_time);
	return py_sim_time;
}


static PyObject	* py_init_simulation(PyObject *self, PyObject *args)
{
	init_simulation();
	RETURN_NONE
}


static PyObject	* py_simulate_next_step(PyObject *self, PyObject *args)
{
	int	r;
	double cu0;
	double tt0;
	int mc0;

	PARSE_FLT2_INT(args, cu0, tt0, mc0)
		
	r = simulate_next_step(1, cu0, tt0, mc0);

	BUILD_INT(py_sim_count, sim_count)
	build_py_flt_array(&py_sim_voltage, sim_voltage, sim_count);
	build_py_flt_array(&py_sim_time, sim_time, sim_count);

	RETURN_INT(r)
}


static PyObject	* py_complete_simulation(PyObject *self, PyObject *args)
{
	int	r;
	
	PARSE_INT(args, r)
	complete_simulation(r);
	RETURN_NONE
}


void	build_variable(void)
{

	sim_count = 0;
	BUILD_INT(py_sim_count, sim_count)
	build_py_flt_array(&py_sim_voltage, sim_voltage, sim_count);
	build_py_flt_array(&py_sim_time, sim_time, sim_count);

	return;
}


static PyMethodDef py_dualfoilMethods[] = {
	{"init_simulation",       py_init_simulation,              METH_VARARGS},
	{"simulate_next_step",    py_simulate_next_step,           METH_VARARGS},
	{"complete_simulation",   py_complete_simulation,          METH_VARARGS},
	{"get_sim_voltage",       py_get_sim_voltage,              METH_VARARGS},
	{"get_sim_time",          py_get_sim_time,                 METH_VARARGS},
	{NULL, NULL},
};

static PyObject *py_dualfoilError;

#ifdef _WIN32
void _declspec(dllexport)
#else
DL_EXPORT(void)
#endif

initpy_dualfoil(void)
{
	PyObject *module, *dict;

	module = Py_InitModule("py_dualfoil", py_dualfoilMethods);
	
	build_variable();

	dict = PyModule_GetDict(module);

	py_dualfoilError = PyErr_NewException("py_dualfoil.error", NULL, NULL);
	PyDict_SetItemString(dict, "error", py_dualfoilError);

	return;
}
