##	Copyright (c) 2004 The Regents of the University of California. All Rights
##	Reserved. Permission to use, copy, modify, and distribute this  software
##	and its documentation for educational, research and non-profit purposes,
##	without fee, and without a written agreement is hereby granted, provided
##	that the above copyright notice, this paragraph and the following two
##	paragraphs appear in all copies. This software program and documentation
##	are copyrighted  by The Regents of the University of California ("The
##	University of California").
##
##	THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
##	ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
##	UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
##	PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
##	THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
##	TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.
##
##	IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
##	DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
##	LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
##	DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
##	THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
##	DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##	WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
##	SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
##	CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
##	ENHANCEMENTS, OR  MODIFICATIONS.

import	math
from	Tkinter import *

eps = 1e-10

xmark_offset = 25
ymark_offset = 15

arrow_offset = 10
left_padding = 5
mark_dx = 5
mark_dy = 5


def	getGridCount(maxvalue, minvalue, grid):
	c = (maxvalue - minvalue) / grid
	cf = math.floor(c)
	if math.fabs(c - cf) < eps:
		return cf
	else:
		return cf + 1
	

def	getGrid(maxvalue, minvalue, dividor):
	grid = float(maxvalue - minvalue) / float(dividor)
	order = math.floor(math.log10(grid))
	unit_grid = grid / math.pow(10, order)
	if unit_grid >= 5.0:	unit_grid = 5.0
	elif unit_grid >= 2.0:	unit_grid = 2.0
	else:	unit_grid = 1.0
	grid = unit_grid * math.pow(10, order)
	if maxvalue != grid * int(maxvalue / grid):
		maxvalue = grid * int(maxvalue / grid + 1)
	minvalue = grid * int(minvalue / grid)
	if math.fabs(maxvalue) < eps:
		maxvalue = 0.
	if math.fabs(minvalue) < eps:
		minvalue = 0.
	return maxvalue, minvalue, grid
	

def	getGridFormat(x):
	grid_order = math.log10(x)
	if grid_order >= 0:
		grid_format = '%d'
	else:
		grid_format = '%%.%df' % (-int(grid_order) + 1)
	return grid_format


class	schart:
	x_axe_offset = -1
	y_axe_offset = 0
	arrow_offset = 20
	dot_dx = dot_dy = 5
	label_y_offset = 10

	grid_color = '#e0e0e0'
	axe_color = '#000000'
	mark_color = '#000000'
	
	def	__init__(self,
				 top_widget,
				 root_canvas,
				 chart_canvas,
				 xmark_canvas,
				 ymark_canvas,
				 chart_canvas_size,
				 xmark_canvas_size,
				 ymark_canvas_size,
				 chart_canvas_anchor, 
				 window_scale,
				 full_scale,
				 #scroll_factors, 
				 n_curve,
				 curve_color,
				 line_width, 
				 xsync_sibling = [],
				 ysync_sibling = []):

		self.top_widget = top_widget
		
		self.root_canvas = root_canvas
		self.chart_canvas = chart_canvas
		self.xmark_canvas = xmark_canvas
		self.ymark_canvas = ymark_canvas

		self.width, self.height = chart_canvas_size
		self.xmark_canvas_width, self.xmark_canvas_height = xmark_canvas_size
		self.ymark_canvas_width, self.ymark_canvas_height = ymark_canvas_size
		
		self.x1, self.y1 = chart_canvas_anchor

		self.x2 = self.x1 + self.width
		self.y2 = self.y1 + self.height

		self.x_left, self.y_bottom, self.x_right, self.y_top = window_scale
		self.x_min, self.y_min, self.x_max, self.y_max = full_scale

		self.n_curve = n_curve
		
		self.curve_color = curve_color[:]
		self.line_width = line_width[:]

		self.xsync_sibling = xsync_sibling[:]
		self.ysync_sibling = ysync_sibling[:]
		
		self.x_axe0 = self.x1 -1
		self.y_axe0 = self.y2 
		self.x_axe1 = self.x2 + xmark_offset + arrow_offset
		self.y_axe1 = self.y1 - ymark_offset - arrow_offset

		self.ymark_yoffset = ymark_offset
		self.ymark_xoffset = self.x1 - left_padding - 2

		self.xmark_xoffset = 1
		self.xmark_yoffset = 0

		self.x_off = 0
		self.y_off = 0

		self.x_data = []
		self.y_data = []
		self.x_point = []
		self.y_point = []

		for curve_index in range(self.n_curve):
			self.x_data.append([])
			self.y_data.append([])
			self.x_point.append([])
			self.y_point.append([])

		self.setScale()

		self.x_left_point = 0
		self.y_top_point = 0

		self.root_tag = []
		self.grid_tag = []
		self.xmark_tag = []
		self.ymark_tag = []
		self.label_tag = []

		self.label = []

		self.chart_tag = []
		for curve_index in range(self.n_curve):
			self.chart_tag.append([])
			self.label_tag.append(None)
			self.label.append('')
		
		self.drawEmpty()

		self.chart_canvas.bind('<Button-1>', self.mouse_click)

		self.chart_canvas.bind('<B1-Motion>', self.mouse_move)

		self.chart_canvas.bind('<Double-Button-1>', self.mouse_doubleclick)
		self.chart_canvas.bind('<Shift-Double-Button-1>', self.mouse_shiftdoubleclick)
		

	def	set_sync_sibling(self, xsync_sibling, ysync_sibling):
		self.xsync_sibling = xsync_sibling[:]
		self.ysync_sibling = ysync_sibling[:]


	def	mouse_shiftdoubleclick(self, event):
		#print 'doubleclicked at', event.x, event.y
		self.zoomx(event.x, 0.5, sync = 1)


	def	mouse_doubleclick(self, event):
		#print 'doubleclicked at', event.x, event.y
		self.zoomx(event.x, 2, sync = 1)
		

	def mouse_click(self, event):
		self.last_x = event.x
		self.last_y = event.y
		#print 'clicked at', event.x, event.y


	def	mouse_move(self, event):
		dx = event.x - self.last_x
		dy = event.y - self.last_y
		#print 'dragged from (%d, %d) to (%d, %d)' % (self.last_x, self.last_y, event.x, event.y)
		self.xscroll(SCROLL, -dx, UNITS, sync = 1)
		self.yscroll(SCROLL, -dy, UNITS, sync = 1)
		self.last_x = event.x
		self.last_y = event.y
	

	def	setScroll(self):
		self.xscroll_factor = float(self.x_max_point) / self.width
		self.yscroll_factor = float(self.y_min_point) / self.height

		self.chart_canvas.config(scrollregion = (0, 0, self.x_max_point, self.y_min_point))
		self.xmark_canvas.config(scrollregion = (0, 0, self.xmark_canvas_width * self.xscroll_factor, self.xmark_canvas_height))
		self.ymark_canvas.config(scrollregion = (0, 0, self.ymark_canvas_width, self.ymark_canvas_height * self.yscroll_factor))
		
	
	def	setScale(self):
		self.resetXGrid()
		self.resetYGrid()
		self.setScroll()


	def	resetScale(self, window_scale, full_scale):
		x_grid, y_grid = self.x_grid, self.y_grid
		x_grid_width, y_grid_height = self.x_grid_width, self.y_grid_height 
		
		x_left, y_bottom, x_right, y_top = self.x_left, self.y_bottom, self.x_right, self.y_top
		x_min, y_min, x_max, y_max = self.x_min, self.y_min, self.x_max, self.y_max
		
		self.x_left, self.y_bottom, self.x_right, self.y_top = window_scale
		self.x_min, self.y_min, self.x_max, self.y_max = full_scale

		self.setScale()

		x_left1 = self.x_left
		x_right1 = self.x_right

		self.x_left = x_left
		self.x_right = x_left + x_right1 - x_left1

		resetX = 0
		resetY = 0

		if (x_min, x_max) != (self.x_min, self.x_max) or \
		   (x_grid, x_grid_width) != (self.x_grid, self.x_grid_width):
			resetX = 1

		#print y_grid, y_grid_height, self.y_grid, self.y_grid_height
		if (y_min, y_max) != (self.y_min, self.y_max) or \
		   (y_grid, y_grid_height) != (self.y_grid, self.y_grid_height):
			resetY = 1

		#print resetX, resetY
		if resetX or resetY: 
			self.clear()

			self.drawEmpty()

			for curve_index in range(self.n_curve):
				if resetX:
					self.rebuildX(curve_index)
				if resetY:
					self.rebuildY(curve_index)
					
				self.drawChart(curve_index)

		x0 = self.getXPos(x_left1)
		y0 = self.getYPos(self.y_top)
		self.scrollPoint(x0, y0, 0, 0)

		return x_left1, self.y_bottom, x_right1, self.y_top
			
		
	# secondary data derived from updated data
	def	resetXGrid(self):
		if self.x_right <= self.x_left:
			print 'Error', self.x_right, self.x_left
			return

		self.x_right, self.x_left, self.x_grid = getGrid(self.x_right, self.x_left, 10)

		self.resetXGridVars()


	def	resetXGridVars(self):		
		self.xsize_factor = float(self.width) / (self.x_right - self.x_left)

		c = getGridCount(self.x_left, self.x_min, self.x_grid)
		self.x_min = self.x_left - c * self.x_grid
		if math.fabs(self.x_min) < eps:
			self.x_min = 0.

		c = getGridCount(self.x_max, self.x_min, self.x_grid)
		self.x_max = self.x_min + c * self.x_grid
		if math.fabs(self.x_max) < eps:
			self.x_max = 0.

		self.x_max_point = self.getXPos(self.x_max)

		self.x_grid_count_max = int(self.x_max / self.x_grid + 0.5)
		self.x_grid_count_min = int(self.x_min / self.x_grid + 0.5)

		self.x_grid_width = self.getXPos(self.x_min + self.x_grid)

		self.xmark_format = getGridFormat(self.x_grid)
		
		self.xscroll_stride_count = self.width / 2
		

	def	resetYGrid(self):
		if self.y_top <= self.y_bottom:
			print 'Error', self.y_top, self.y_bottom
			return

		self.y_top, self.y_bottom, self.y_grid = getGrid(self.y_top, self.y_bottom, 5)

		self.resetYGridVars()


	def	resetYGridVars(self):
		self.ysize_factor = float(self.height) / (self.y_top - self.y_bottom)

		c = getGridCount(self.y_bottom, self.y_min, self.y_grid)
		self.y_min = self.y_bottom - c * self.y_grid
		if math.fabs(self.y_min) < eps:
			self.y_min = 0.

		c = getGridCount(self.y_max, self.y_min, self.y_grid)
		self.y_max = self.y_min + c * self.y_grid
		if math.fabs(self.y_max) < eps:
			self.y_max = 0.

		self.y_max_point = self.getYPos(self.y_max)
		self.y_min_point = self.getYPos(self.y_min)

		self.y_grid_count_max = int(self.y_max / self.y_grid + 0.5)
		self.y_grid_count_min = int(self.y_min / self.y_grid + 0.5)

		self.y_grid_height = self.getYPos(self.y_max - self.y_grid)

		self.ymark_format = getGridFormat(self.y_grid)
		

	# translate a value to Y
	def	getYPos(self, value):
		return self.y_off + int((self.y_max - value) * self.ysize_factor)


	def	getYValue(self, y):
		return self.y_max - (y - self.y_off) / self.xsize_factor


	# translate a value to X
	def	getXPos(self, value):
		return self.x_off + int((value - self.x_min) * self.xsize_factor)


	def	getXValue(self, x):
		return (x - self.x_off) / self.xsize_factor + self.x_min


	# axe of Y
	def	drawYAxe(self):
		id = self.root_canvas.create_line(self.x_axe0, self.y_axe0, self.x_axe0, self.y_axe1, fill = self.axe_color, arrow = 'last')
		self.root_tag.append(id)


	# axe of X
	def	drawXAxe(self):
		id = self.root_canvas.create_line(self.x_axe0, self.y_axe0, self.x_axe1, self.y_axe0, fill = self.axe_color, arrow = 'last')
		self.root_tag.append(id)


	# draw time grid
	def	drawGridX(self):
		i = self.x_grid_count_min
		while 1:
			x = self.getXPos(i * self.x_grid)
			if x > self.x_max_point:
				break;
			x -= 1
			id = self.chart_canvas.create_line(x, 0, x, self.y_min_point, fill = self.grid_color)
			self.grid_tag.append(id)

			id = self.xmark_canvas.create_line(self.xmark_xoffset + x, self.xmark_yoffset, self.xmark_xoffset + x, self.xmark_yoffset + mark_dy, fill = self.mark_color)
			self.xmark_tag.append(id)

			s = self.xmark_format % (i * self.x_grid)
			id = self.xmark_canvas.create_text(self.xmark_xoffset + x, self.xmark_yoffset + mark_dy, text = s, fill = self.mark_color, anchor = NW)
			self.xmark_tag.append(id)

			i += 1
			

	# draw power grid
	def	drawGridY(self):
		i = self.y_grid_count_min
		while 1:
			y = self.getYPos(i * self.y_grid)
			if y < self.y_max_point:
				break
			id = self.chart_canvas.create_line(0, y, self.x_max_point, y, fill = self.grid_color)
			self.grid_tag.append(id)

			id = self.ymark_canvas.create_line(self.ymark_xoffset, self.ymark_yoffset + y, self.ymark_xoffset - mark_dx, self.ymark_yoffset + y, fill = self.mark_color)
			self.ymark_tag.append(id)

			s = self.ymark_format % (i * self.y_grid)			
			id = self.ymark_canvas.create_text(self.ymark_xoffset - mark_dx, self.ymark_yoffset + y, text = s, fill = self.mark_color, anchor = SE)
			self.ymark_tag.append(id)
			
			i += 1
			

	def	drawEmpty(self):
		self.drawGridX()
		self.drawGridY()
		
		self.drawXAxe()
		self.drawYAxe()

		x0 = self.getXPos(self.x_left)
		y0 = self.getYPos(self.y_top)

		self.scrollPoint(x0, y0, 0, 0)

		for curve_index in range(self.n_curve):
			self.drawLabel(self.label[curve_index], curve_index)


	def	scrollPoint(self, x, y, xref, yref):
		self.scrollXPoint(x, xref)
		self.scrollYPoint(y, yref)


	def	scrollXPoint(self, x, xref):
		step = x - self.x_left_point - xref
		self.xscroll(SCROLL, step, UNITS, sync = 0)
		

	def	scrollYPoint(self, y, yref):
		step = y - self.y_top_point - yref
		self.yscroll(SCROLL, step, UNITS, sync = 0)


	def	xscroll(self, flag0, step0, unit0 = UNITS, sync = 1):
		if flag0 == SCROLL and unit0 == PAGES:
			step = int(float(step0) * self.width)
		elif flag0 == SCROLL and unit0 == UNITS:
			step = int(step0)
		elif flag0 == MOVETO and unit0 == UNITS:
			step = int(float(step0) * self.xscroll_factor * self.width) - self.x_left_point
		else:
			print 'Undefined flag =', flag0, 'step = ', step0, 'unit = ', unit0
			return

		self.x_left_point += step
		if self.x_left_point < 0:
			step -= self.x_left_point
			self.x_left_point = 0
		elif self.x_left_point >= self.x_max_point - self.width:
			step -= (self.x_left_point - self.x_max_point + self.width)
			self.x_left_point = self.x_max_point - self.width
		
		self.chart_canvas.xview(SCROLL, step, UNITS)
		self.xmark_canvas.xview(SCROLL, step, UNITS)

		self.chart_canvas.xview(SCROLL, 0, UNITS)
		self.xmark_canvas.xview(SCROLL, 0, UNITS)

		if sync == 1:
			for c in self.xsync_sibling:
				c.xscroll(flag0, step0, unit0, sync = 0)
				

	def	yscroll(self, flag0, step0, unit0 = UNITS, sync = 1):
		if flag0 == SCROLL and unit0 == PAGES:
			step = int(float(step0) * self.height)
		elif flag0 == SCROLL and unit0 == UNITS:
			step = int(step0)
		elif flag0 == MOVETO and unit0 == UNITS:
			step = int(float(step0) * self.yscroll_factor * self.height) - self.y_top_point
		else:
			print 'Undefined flag =', flag0, 'step = ', step0, 'unit = ', unit0
			return

		self.y_top_point += step
		if self.y_top_point < 0:
			step -= self.y_top_point
			self.y_top_point = 0
		elif self.y_top_point >= self.y_min_point - self.height:
			step -= (self.y_top_point - self.y_min_point + self.height)
			self.y_top_point = self.y_min_point - self.height

		self.chart_canvas.yview(SCROLL, step, UNITS)
		self.ymark_canvas.yview(SCROLL, step, UNITS)

		if sync == 1:
			for c in self.ysync_sibling:
				c.yscroll(flag0, step0, unit0, sync = 0)

		
	# update new set of data, raw data only
	def	updateData(self, x_data, y_data, curve_index = 0):
		self.x_data[curve_index] = x_data[:]
		self.y_data[curve_index] = y_data[:]

		self.rebuildX(curve_index)
		self.rebuildY(curve_index)
		
		self.drawChart(curve_index)

		#self.chart_canvas.update()
		

	def	rebuildX(self, curve_index = 0):
		self.x_point[curve_index] = map(self.getXPos, self.x_data[curve_index])


	def	rebuildY(self, curve_index = 0):		
		self.y_point[curve_index] = map(self.getYPos, self.y_data[curve_index])
		
		
	def	appendData(self, x_data, y_data, curve_index = 0):
		self.x_data[curve_index].extend(x_data)
		self.y_data[curve_index].extend(y_data)
		
		x_point = [self.x_point[curve_index][-1]]
		y_point = [self.y_point[curve_index][-1]]

		for i in range(len(x_data)):
			x = x_data[i]
			y = y_data[i]
			x_point.append(self.getXPos(x))
			y_point.append(self.getYPos(y))

		for i in range(1, len(x_point)):		
			id = self.chart_canvas.create_line(x_point[i-1], y_point[i-1], x_point[i], y_point[i], fill = self.curve_color[curve_index], width = self.line_width[curve_index])
			self.chart_tag[curve_index].append(id)

		x_point.pop(0)
		y_point.pop(0)
		
		self.x_point[curve_index].extend(x_point)
		self.y_point[curve_index].extend(y_point)

		if self.x_point[curve_index][-1] >= self.x_left_point + self.width:
			step = int((self.x_point[curve_index][-1] - (self.x_left_point + self.width)) / self.xscroll_stride_count + 1) * self.xscroll_stride_count
			self.xscroll(SCROLL, step, UNITS, sync = 0)

		if self.y_point[curve_index][-1] >= self.y_top_point + self.height:
			step = int((self.y_point[curve_index][-1] - (self.y_top_point + self.height)) / self.y_grid_height + 1) * self.y_grid_height
			self.yscroll(SCROLL, step, UNITS, sync = 0)
		elif self.y_point[curve_index][-1] <= self.y_top_point:
			step = -int((self.y_top_point - self.y_point[curve_index][-1]) / self.y_grid_height + 1) * self.y_grid_height
			self.yscroll(SCROLL, step, UNITS, sync = 0)

		#self.chart_canvas.update()
			

	# power chart	
	def	drawChart(self, curve_index = 0):
		for i in range(1, len(self.x_point[curve_index])):
			id = self.chart_canvas.create_line(self.x_point[curve_index][i-1], self.y_point[curve_index][i-1], self.x_point[curve_index][i], self.y_point[curve_index][i], fill = self.curve_color[curve_index], width = self.line_width[curve_index])
			self.chart_tag[curve_index].append(id)

	def	drawLabel(self, label, curve_index = 0):
		if self.label_tag[curve_index] != None:
			self.clearLabel(curve_index)
		
		x = self.x1 + curve_index * (self.width / self.n_curve) + self.width / self.n_curve / 2
		y = self.label_y_offset
		
		id = self.root_canvas.create_text(x, y, text = label, fill = self.curve_color[curve_index], anchor = N)
		self.label_tag[curve_index] = id
		self.label[curve_index] = label


	def	rollBack(self):
		x0 = self.getXPos(self.x_left)
		y0 = self.getYPos(self.y_top)
		self.scrollPoint(x0, y0, 0, 0)


	def	clear(self):
		self.clearAllChart()
		self.clearGrid()
		self.clearRoot()
		self.clearXmark()
		self.clearYmark()


	def	clearLabel(self, curve_index = 0):
		self.root_canvas.delete(self.label_tag[curve_index])
		self.label_tag[curve_index] = None


	def	clearAllChart(self):
		for i in range(self.n_curve):
			self.clearChart(i)
		self.scrollPoint(0, 0, 0, 0)
			

	def	clearChart(self, curve_index = 0):
		for tag in self.chart_tag[curve_index]:
			self.chart_canvas.delete(tag)
		self.chart_tag[curve_index] = []
		self.clearLabel(curve_index)


	def	clearAllData(self):
		for i in range(self.n_curve):
			self.clearData(i)

	def	clearData(self, curve_index = 0):
		self.clearChart(curve_index)
		
		self.label[curve_index] = ''
		
		self.x_data[curve_index] = []
		self.y_data[curve_index] = []
		self.x_point[curve_index] = []
		self.y_point[curve_index] = []

		
	def	clearGrid(self):
		for tag in self.grid_tag:
			self.chart_canvas.delete(tag)
		self.grid_tag = []

		
	def	clearRoot(self):
		for tag in self.root_tag:
			self.root_canvas.delete(tag)
		self.root_tag = []
		#self.clearLabel()


	def	clearXmark(self):
		for tag in self.xmark_tag:
			self.xmark_canvas.delete(tag)
		self.xmark_tag = []

	def	clearYmark(self):
		for tag in self.ymark_tag:
			self.ymark_canvas.delete(tag)
		self.ymark_tag = []


	def	zoomx(self, xref, factor, sync = 1):
		if sync == 1:
			self.top_widget.setWait()
		
		xvalue = self.getXValue(xref + self.x_left_point)
		
		self.x_right /= factor
		self.x_left /= factor

		self.resetScale((self.x_left, self.y_bottom, self.x_right, self.y_top), (self.x_min, self.y_min, self.x_max, self.y_max))
		
		xnew = self.getXPos(xvalue)
		self.scrollXPoint(xnew, xref)

		if sync:
			for c in self.xsync_sibling:
				c.zoomx(xref, factor, sync = 0)

			self.top_widget.clearWait()				


	def	repaintCurve(self, curve_index, new_curve_color, new_line_width):
		if self.curve_color[curve_index] != new_curve_color or self.line_width[curve_index] != new_line_width:
			self.curve_color[curve_index] = new_curve_color
			self.line_width[curve_index] = new_line_width
			
			self.clearChart(curve_index)
			self.drawChart(curve_index)
			self.drawLabel(self.label[curve_index], curve_index)
			
		
	def	updateColorWidth(self, curve_color, line_width):
		for curve_index in range(self.n_curve):
			self.repaintCurve(curve_index, curve_color[curve_index], line_width[curve_index])

