/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


/*
  send.c
  written by Richard Gordon
  last modified 2/27/03

  This program sends n packets on a specific interface each containing
  a specified sequence of nibbles indicated by their hex values.
  It is called by

  send <n> <interface> <string of hex digits>

  where <n> is the number of packets to send,
  <interface> is "eth0", "eth1", etc.,
  and <string of hex digits> is a sequence of hex characters indicating
  the packet contents to send. Each character is packed into one nibble
  of the packet. If there are an odd number of characters a 0x0 is
  appended.
*/

#define BUFFER_LEN 1600
#define ETH_P_ALL 0x0003
#define ETH_P_IP  0x0800

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <unistd.h>
#include <net/ethernet.h>
#include <errno.h>

int
hex_char_to_binary( char c );

static int in_ether (char *bufp, unsigned char *addr);

int
main( int argc, char* argv[] )
{
  struct sockaddr address;
  //struct ether_addr *e_addr;
  unsigned char addr_buf[BUFFER_LEN];
  unsigned char buffer[BUFFER_LEN];
  int sock;
  int packet_count;
  int octet_count;
  int i;
  char *interface;

  /* the call must be of the form send <n> <interface> <string of hex digitss>*/
  if (argc != 4)
  {
    fprintf( stderr, "usage: send <n> <interface> <string of hex digits>\n" );
    exit(1);
  }
  packet_count = atoi( argv[1] );
  interface = argv[2];

/*
  for (i = 0; i < ETHER_ADDR_LEN; i ++)
  {
    addr_buf[ i ] = hex_char_to_binary( argv[ 2 ][ 2*i ] ) << 4 | hex_char_to_binary( argv[ 2 ][ 2*i+1 ] ) & 0377;
    //printf("%c%c -> %x\n", argv[2][2*i], argv[2][2*i+1], buffer[i]);
  }
*/
 
  //in_ether(argv[2], addr_buf);
  //e_addr = ether_aton(argv[2]);
  
  //strncpy(e_addr.octet, addr_buf, ETHER_ADDR_LEN);
  
  //interface = ether_ntoa(e_addr);

  octet_count = (strlen( argv[ 3 ] )+1) / 2 ;
  
  /* pack the hex digits into the buffer two per octet */
  for (i=0; i<octet_count; ++i)
    buffer[ i ] = hex_char_to_binary( argv[ 3 ][ 2*i ] ) << 4 | hex_char_to_binary( argv[ 3 ][ 2*i+1 ] );

  /* build address data structure */
  memset( &address, 0, sizeof( address ) );
  address.sa_family = AF_NDRV;
  strncpy( address.sa_data, interface, sizeof(address.sa_data) );

  if ((sock = socket( AF_NDRV, SOCK_RAW, 0))<0)
  {
    perror( "send: socket" );
    exit(1);
  }

  fprintf(stdout, "Sending %d packets to addr: %s data: %s\n", packet_count, interface, buffer);

  /* main loop */
  for (i=0; i<packet_count; i++ )
  {
    sendto( sock, buffer, octet_count, 0, &address, sizeof( address ) );
  }
  close( sock );

}

/* convert hex digit to corresponding binary value */
int
hex_char_to_binary( char c )
{
  int result;
	
  if (isxdigit( c ))
  {
	if (isdigit( c ) )
      result = c - '0';
    else
      result = toupper( c ) - 'A' + 10;
  }
  else if (c != 0) // in case the hex_digit string has an odd length
  {
    fprintf( stderr, "non hex digiti found in string\n" );
    exit(1);
  }
	
	return result;
}

/* Input an Ethernet address and convert to binary. */
static int in_ether (char *bufp, unsigned char *addr)
{
    char c, *orig;
    int i;
        unsigned char *ptr = addr;
    unsigned val;

    i = 0;
    orig = bufp;
    while ((*bufp != '\0') && (i < ETHER_ADDR_LEN)) {
        val = 0;
        c = *bufp++;
        if (isdigit(c))
            val = c - '0';
        else if (c >= 'a' && c <= 'f')
            val = c - 'a' + 10;
        else if (c >= 'A' && c <= 'F')
            val = c - 'A' + 10;
        else {
#ifdef DEBUG
            fprintf(stderr, "in_ether(%s): invalid ether address!\n", orig);
#endif
            errno = EINVAL;
            return (-1);
        }
        val <<= 4;
        c = *bufp;
        if (isdigit(c))
            val |= c - '0';
        else if (c >= 'a' && c <= 'f')
            val |= c - 'a' + 10;
        else if (c >= 'A' && c <= 'F')
            val |= c - 'A' + 10;
        else if (c == ':' || c == 0)
            val >>= 4;
        else {
#ifdef DEBUG
            fprintf(stderr, "in_ether(%s): invalid ether address!\n", orig);
#endif
            errno = EINVAL;
            return (-1);
        }
        if (c != 0)
            bufp++;
        *ptr++ = (unsigned char) (val & 0377);
        i++;

        /* We might get a semicolon here - not required. */
        if (*bufp == ':') {
            if (i == ETHER_ADDR_LEN) {
                    ;           /* nothing */
            }
            bufp++;
        }
    }
    return (0);
} /* in_ether */


