/*
Copyright (c) 2004 The Regents of the University of California. All Rights
Reserved. Permission to use, copy, modify, and distribute this  software
and its documentation for educational, research and non-profit purposes,
without fee, and without a written agreement is hereby granted, provided
that the above copyright notice, this paragraph and the following two
paragraphs appear in all copies. This software program and documentation
are copyrighted  by The Regents of the University of California ("The
University of California").

THE SOFTWARE PROGRAM AND DOCUMENTATION ARE SUPPLIED "AS IS," WITHOUT ANY
ACCOMPANYING SERVICES FROM THE UNIVERSITY OF CALFORNIA. FURTHERMORE, THE
UNIVERSITY OF CALIFORNIA DOES NOT WARRANT THAT THE OPERATION OF THE
PROGRAM  WILL BE UNINTERRUPTED OR ERROR-FREE. THE END-USER UNDERSTANDS
THAT THE PROGRAM  WAS DEVELOPED FOR RESEARCH PURPOSES AND IS ADVISED NOT
TO RELY EXCLUSIVELY  ON THE PROGRAM FOR ANY REASON.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
DOCUMENTATION,  EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGES. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
DISCLAIMS ANY  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
SOFTWARE PROVIDED  HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR  MODIFICATIONS.
*/


#include <stdlib.h>
#include "B_ethernet.h"

sock_type	sock;
char	device[20];

#define	PACKET_LEN 256

int main(int argc, char **argv)
{
	int	hex_char_count;
	unsigned char	send_buf[PACKET_LEN];
	unsigned char	recv_buf[PACKET_LEN];
	int	n;
	struct	timeval	tv, *p_tv;
	double	timeout = 0.;
	int	blocking;
	int i;

	if ((argc < 4) || (argc > 1) && (strcmp(argv[1], "-h") == 0))
 	{
		fprintf(stderr, "Usage: %s <device> <n> <hex string> [timeout, 0=blocking, -1=non_blocking]\n", argv[0]);
 		exit(1);
	}

	strcpy(device, argv[1]);

	n = atoi(argv[2]);

 	hex_char_count = (strlen(argv[3]) + 1) / 2 ;
 
 	for (i = 0; i < hex_char_count; ++i)
 		send_buf[i] = hex_char_to_binary(argv[3][2 * i]) << 4 | hex_char_to_binary(argv[3][2 * i + 1]);

	if (argc > 4)
		timeout = atof(argv[4]);

	sock = open_socket(device, timeout);

#ifdef	_WIN32
	if (sock == NULL)
#else
	if (sock < 0)
#endif
	{
		printf("Error: cannot open device %s\n", device);
		exit(2);
	}

	if (timeout > 0.)
	{
		tv.tv_sec = (int)timeout;
		tv.tv_usec = (int)((timeout - tv.tv_sec) * 1.e6);
		p_tv = &tv;
		blocking = 1;
	}
	else if (timeout == 0.)
	{
		p_tv = NULL;
		blocking = 1;
	}
	else
	{
		p_tv = NULL;
		blocking = 0;
	}

	set_blocking(sock, blocking);
	set_promiscuous_mode(sock, device, 1);

	signal(SIGINT, interrupt_handler);
	
	for (i = 0; i < n; i ++)
	{
		printf("send count = %d\n", i + 1);
		send_to_socket(sock, device, send_buf, PACKET_LEN);

		//int r = 0;
		//while (r <= 0)
		//	r = recv_from_socket(sock, device, recv_buf, PACKET_LEN);
		recv_from_socket(sock, device, recv_buf, PACKET_LEN, p_tv);
	}
	
	interrupt_handler(0);

	return 0;
}
